var app = angular.module('PasswordMonitor', []);
app.controller('BackgroundCtrl', function (HashingService, MonitorService, IdbService, TimerService, IconService,
                                           DataUploadService) {

    /**
     * Listen to installations and create user-unique salts
     */
    chrome.runtime.onInstalled.addListener(function () {
        HashingService.getSalt().then(function (hasSalt) {
            if (!hasSalt) {
                chrome.tabs.create({url: chrome.extension.getURL('html/settings.html'), active: true});
            }
        });
    });

    chrome.runtime.onConnect.addListener(function (port) {
        if (port && port['name'] === 'uploadData') {
            DataUploadService.uploadData(port);
        }
    });

    chrome.runtime.onMessage.addListener(function (message, sender, callback) {

        if (message) {
            switch (message.type) {

                case 'newPassword':
                    if (message.url && message.username && message.password && message.time && message.timeSpent) {
                        MonitorService.newPassword(
                            message.url,
                            message.username,
                            message.password,
                            message.time,
                            message.timeSpent);
                    }
                    break;

                case 'getPasswordMetrics':
                    IdbService.getAllValues('metrics').then(function (metrics) {
                        callback({passwordMetrics: metrics});
                    });
                    return true;

                case 'getCredentialUses':
                    IdbService.getAllValues('credentialUses').then(function (credentialUses) {
                        callback({credentialUses: credentialUses});
                    });
                    return true;

                case 'deleteCheckedEntries':
                    IdbService.deleteCheckedEntries(message.credentialUses, callback);
                    return true;

                case 'deleteEntriesFromUrl':
                    if (message.url) {
                        IdbService.deleteEntriesFromUrl(message.url);
                    }
                    break;

                case 'startTimer':
                    if (message.time) {
                        TimerService.startTimer(message.time);
                        IconService.suspend();
                    }
                    break;

                case 'stopTimer':
                    TimerService.stopTimer();
                    IconService.resume();
                    break;

                case 'getTime':
                    callback(TimerService.getAll());
                    return true;

                case 'openChromeExtensions':
                    chrome.tabs.create({url: "chrome://extensions/"});
                    break;

                case 'getSalt':
                    HashingService.getSalt().then(function (salt) {
                        callback(salt);
                    });
                    return true;

                case 'setEmail':
                    if (message.email) {
                        HashingService.hashThis(message.email).then(function (salt) {
                            chrome.storage.local.set({'email': message.email, 'salt': salt});
                        });
                    }
                    break;

                case 'setEmailNH':
                    if (message.email) {
                        HashingService.hashThis(message.email).then(function (salt) {
                            chrome.storage.local.set({'email': message.email});
                        });
                    }
                    break;

                case 'hasCollectedFromThisSite':
                    if (message.url) {
                        IdbService.hasCollectedFromThisSite(message.url).then(function (has) {
                            callback(has);
                        });
                    }
                    return true;

                case 'hashThis':
                    if (message.cleartext) {
                        HashingService.hashThis(message.cleartext).then(function (hash) {
                            callback(hash);
                        });
                    }
                    return true;

                case 'setConcurrency':
                    if (message.numberOfWorkers) {
                        HashingService.setNumberOfWorkers(message.numberOfWorkers);
                    }
                    break;

                case 'getConcurrency':
                    callback(HashingService.getNumberOfWorkers());
            }
        }
    });
});
