var app = angular.module('PasswordMonitor', ['ui.bootstrap', 'easypiechart']);
app.controller('BrowserActionCtrl', function ($scope, $window, $q, TimerService, IconService) {

    $scope.blur = function () {
        $("button.btn").blur();
    };

    /**
     * Timer controls
     */
    $scope.time = {
        options: {
            size: 150
        },
        isSuspended: TimerService.isSuspended,
        minutes: TimerService.getMinutes,
        seconds: TimerService.getSeconds,
        percent: TimerService.getPercent
    };
    chrome.runtime.sendMessage({'type': 'getTime'}, function (time) {
        if (time) {
            $scope.collapseStyle = {"transition": "none"};
            TimerService.startTimer(time);
        }
    });
    $scope.startTimer = function (seconds) {
        var time = {
            startTime: seconds,
            totalSeconds: seconds
        };
        chrome.runtime.sendMessage({'type': 'startTimer', 'time': time});
        TimerService.startTimer(time);
    };
    $scope.stopTimer = function () {
        chrome.runtime.sendMessage({'type': 'stopTimer'});
        $scope.collapseStyle = {
            "transition-timing-function": "ease",
            "transition-duration": ".35s",
            "transition-property": "height, visibility"
        };
        TimerService.stopTimer();
    };

    $scope.openSettings = function () {
        chrome.tabs.create({url: 'chrome://extensions/?options=' + chrome.runtime.id});
    };

    function getUrl() {
        var deferred = $q.defer();
        chrome.tabs.query({currentWindow: true, active: true}, function (tabs) {
            var url = tabs[0].url;
            var canonicalUrl = url.split("/")[2];
            deferred.resolve(canonicalUrl);
        });
        return deferred.promise;
    }

    /**
     * To begin with, find out whether we are collecting from this website currently or if we have in the past
     */
    getUrl().then(function (url) {
        chrome.runtime.sendMessage({'type': 'hasCollectedFromThisSite', 'url': url}, function (has) {
            $scope.$apply(function () {
                $scope.hasCollectedFromThisSite = has;
            });
        });
        chrome.storage.local.get('exclusionList', function (result) {
            $scope.$apply(function () {
                if (angular.isDefined(result['exclusionList'])) {
                    $scope.isCollecting = result['exclusionList'].indexOf(url) === -1;
                }
                else {
                    $scope.isCollecting = true;
                }
            });
        });
    });

    $scope.toggleCollecting = function () {
        getUrl().then(function (url) {
            chrome.storage.local.get('exclusionList', function (result) {
                if (angular.isDefined(result['exclusionList']) && $scope.isCollecting) {
                    result['exclusionList'].push(url);
                }
                else if ($scope.isCollecting) {
                    result['exclusionList'] = [url];
                }
                else {
                    var index = result['exclusionList'].indexOf(url);
                    result['exclusionList'].splice(index, 1);
                }
                chrome.storage.local.set({'exclusionList': result['exclusionList']});
                $scope.$apply(function () {
                    $scope.isCollecting = !$scope.isCollecting;
                });
                $scope.isCollecting ? IconService.notExcluded() : IconService.excluded();
            });
        });
    };

    $scope.deleteThisSite = function () {
        getUrl().then(function (url) {
            chrome.runtime.sendMessage({'type': 'deleteEntriesFromUrl', 'url': url});
        });
        $window.close();
    };
});
