var app = angular.module('PasswordMonitor');
app.constant('serverRootUrl', 'https://study.cs.ucl.ac.uk/password-gb/web/');
app.factory('DataUploadService', function ($q, $http, IdbService, serverRootUrl) {

    function getEmail() {
        var deferred = $q.defer();
        chrome.storage.local.get('email', function (result) {
            if (result['email']) {
                deferred.resolve(result['email']);
            }
            else {
                deferred.reject();
            }
        });
        return deferred.promise;
    }

    return {

        uploadData: function (port) {
            var tables;
            if (port && port.postMessage) {
                port.postMessage({inProgress: true, stage: 1, text: 'Retrieving data from tables'});
                IdbService.getAllTables().then(function (retrievedTables) {
                    tables = retrievedTables;
                    port.postMessage({inProgress: true, stage: 2, text: 'Getting login status'});
                    return $q.all({
                        response: $http.get(serverRootUrl + 'a/login'),
                        email: getEmail()
                    });
                }).then(function (result) {
                    var response = angular.element(result.response.data);
                    var token, inputs = response.find('input');
                    for (var x = 0; x < inputs.length; x++) {
                        if (inputs[x].name === "_csrf_token") {
                            token = inputs[x].value;
                        }
                    }
                    port.postMessage({inProgress: true, stage: 3, text: 'Logging in as: ' + result.email});
                    return $http({
                        method: 'POST',
                        url: serverRootUrl + 'a/login_check',
                        headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                        transformRequest: function (obj) {
                            var str = [];
                            for (var p in obj) {
                                if (obj.hasOwnProperty(p)) {
                                    str.push(encodeURIComponent(p) + "=" + encodeURIComponent(obj[p]));
                                }
                            }
                            return str.join("&");
                        },
                        data: {
                            _username: result.email,
                            _password: result.email,
                            '_remember_me': 'on',
                            '_csrf_token': token
                        }
                    });
                }).then(function () {
                    var xhr = new XMLHttpRequest();
                    xhr.open('POST', serverRootUrl + 'a/1/upload_direct', true);
                    var blob = new Blob([angular.toJson(tables)], {type: 'application/json'});
                    xhr.onload = function () {
                        var response = angular.fromJson(xhr['response']);
                        if (response['DataLength'] === blob.size && response['DirectUpload'] === 'Success') {
                            port.postMessage({success: true});
                        }
                        else {
                            port.postMessage({success: false});
                        }
                        port.disconnect();
                    };
                    xhr.upload.onprogress = function (event) {
                        port.postMessage({inProgress: true, percentage: 100 * event['loaded'] / event['total']});
                    };
                    xhr.onerror = function (event) {
                        port.postMessage({success: false, errorCode: event.code, error: event.statusText});
                        port.disconnect();
                    };
                    port.postMessage({inProgress: true, stage: 4, text: 'Sending data'});
                    xhr.send(blob);
                }).catch(function (error) {
                    port.postMessage({success: false, errorCode: error.status, error: error.statusText});
                    port.disconnect();
                });
            }
        }
    }
});
