angular.module('PasswordMonitor').factory('IconService', function (TimerService) {

    var isSuspended = TimerService.isSuspended;

    function determineWhetherInExclusionList(url) {
        chrome.storage.local.get('exclusionList', function (result) {
            if (angular.isDefined(result['exclusionList'])) {
                var canonicalUrl = url.split("/")[2];
                for (var i = 0; i < result['exclusionList'].length; i++) {
                    if (result['exclusionList'][i] === canonicalUrl) {
                        setExclusionListBadge(true);
                        return;
                    }
                }
                setExclusionListBadge(false);
            }
        });
    }

    chrome.tabs.onUpdated.addListener(function (tabId, changeInfo, tab) {
        if (!isSuspended() && tab['currentWindow'] && tab['active'] && changeInfo['url']) {
            determineWhetherInExclusionList(changeInfo['url']);
        }
    });

    chrome.tabs.onActivated.addListener(function (activeInfo) {
        if (!isSuspended() && activeInfo && activeInfo['tabId']) {
            var tabId = activeInfo['tabId'];

            chrome.tabs.get(tabId, function (tab) {
                if (!chrome.runtime.lastError && tab && tab['url']) {
                    determineWhetherInExclusionList(tab['url']);
                }
                else if (chrome.runtime.lastError) {
                    console.error(chrome.runtime.lastError.message);
                }
            });
            if (chrome.runtime.lastError) {
                console.error(chrome.runtime.lastError.message);
            }
        }
    });

    function setExclusionListBadge(inList) {
        if (inList) {
            chrome.browserAction.setBadgeBackgroundColor({color: "#FF0"});
            chrome.browserAction.setBadgeText({text: "Stop"});
        }
        else {
            chrome.browserAction.setBadgeText({text: ""});
        }
    }

    return {

        suspend: function () {
            chrome.browserAction.setBadgeText({text: "OFF"});
            chrome.browserAction.setBadgeBackgroundColor({color: "#F00"});
            chrome.browserAction.setIcon({
                path: {
                    '19': '../image/icon19red.png',
                    '38': '../image/icon38red.png'
                }
            });
        },

        resume: function () {
            // Put icon back for sure, but let's check whether the current tab is excluded
            chrome.browserAction.setIcon({
                path: {
                    '19': '../image/icon19.png',
                    '38': '../image/icon38.png'
                }
            });
            chrome.browserAction.setBadgeText({text: ""});
            chrome.tabs.query({currentWindow: true, active: true}, function (tabs) {
                if (tabs && tabs[0]) {
                    var url = tabs[0].url;
                    determineWhetherInExclusionList(url);
                }
            });
        },

        excluded: function () {
            setExclusionListBadge(true);
        },

        notExcluded: function () {
            setExclusionListBadge(false);
        }
    }
});