angular.module('PasswordMonitor').factory('ManglingService', function () {

    var substitutions = [
        ['@', 'a', 'A'],
        ['!', 'i', 'I', '\\|', 'l', 'L', '1'],
        ['o', 'O', '0'],
        ['\\$', 's', 'S', '5'],
        ['3', 'e', 'E'],
        ['9', '6', 'g', 'G'],
        ['8', 'B', 'b'],
        ['\"', '2'],
        ['£', '3'],
        ['\\$', '€', '4'],
        ['%', '5'],
        ['\\^', '6'],
        ['&', '7'],
        ['\\*', '8'],
        ['\\(', '9'],
        ['\\)', '0'],
        ['-', '_', '–'],
        ['\\(', '\\{', '\\['],
        ['\\)', '\\}', '\\]'],
        [';', ':'],
        ['@', '\''],
        ['~', '#'],
        ['<', ','],
        ['>', '\\.'],
        ['\\?', '\/'],
        ['\\|', '\\\\']
    ];
    var symbols = ['!', "+", '=', '?', '/', '\\', '#', '%', '&', '*', ';', ':'];

    function removeDuplication(vars) {
        var noDuplicateVars = [];
        search:
            for (var i1 = 0; i1 < vars.length; i1++) {
                for (var i2 = 0; i2 < noDuplicateVars.length; i2++) {
                    if (vars[i1]['variation'] === noDuplicateVars[i2]['variation']) {
                        continue search;
                    }
                }
                noDuplicateVars.push(vars[i1]);
            }
        return noDuplicateVars;
    }

    function incrementNumbers(password) {
        var vars = [{
            variation: password,
            reason: []
        }];
        var at = 0;
        var match;
        while (match = password.substr(at).match(/\d+/)) {
            var newVars = [];
            for (var n = 0; n < vars.length; n++) {
                var pre = vars[n]['variation'].substr(0, at);
                var toReplace = parseInt(match[0]);
                var rest = vars[n]['variation'].substr(at);
                var replaced = rest.replace(toReplace, toReplace + 1);
                newVars.push({
                    variation: pre + replaced,
                    reason: ['incremented number']
                });
            }
            vars = vars.concat(newVars);
            at += match.index + match[0].length;
        }
        return vars;
    }

    function commonSubstitutions(vars) {
        var at = 0;
        var match;
        var newVars = [];
        for (var s1 = 0; s1 < substitutions.length; s1++) {
            var regex = new RegExp(substitutions[s1].join('|'));
            for (var v = 0; v < vars.length; v++) {
                while (match = vars[v]['variation'].substr(at).match(regex)) {
                    var pre = vars[v]['variation'].substr(0, at);
                    var rest = vars[v]['variation'].substr(at);
                    for (var s2 = 0; s2 < substitutions[s1].length; s2++) {
                        if (substitutions[s1][s2].replace("\\", "") !== match[0]) {
                            var replaced = rest.replace(match[0], substitutions[s1][s2].replace("\\", ""));
                            newVars.push({
                                variation: pre + replaced,
                                reason: vars[v]['reason'].concat(['common substitution'])
                            });
                        }
                    }
                    at += match.index + match[0].length;
                }
                at = 0;
            }
        }
        return removeDuplication(vars.concat(newVars));
    }

    function appendSymbols(vars) {
        for (var s = 0; s < symbols.length; s++) {
            vars.push({
                variation: vars[0]['variation'] + symbols[s],
                reason: ['append symbol']
            });
        }
        return vars;
    }

    function appendCurrentYear(vars) {
        var newVars = [];
        for (var v = 0; v < vars.length; v++) {
            newVars.push({
                variation: vars[v]['variation'] + '15',
                reason: vars[v]['reason'].concat(['append current year'])
            });
            newVars.push({
                variation: vars[v]['variation'] + '2015',
                reason: vars[v]['reason'].concat(['append current year'])
            });
        }
        return removeDuplication(vars.concat(newVars));
    }

    return {
        mangle: function (password) {
            if (password) {
                var vars = incrementNumbers(password);
                vars = commonSubstitutions(vars);
                vars = appendSymbols(vars);
                vars = appendCurrentYear(vars);
                vars.shift(); // The first element is the password itself, so let's remove it
                return vars;
            }
        }
    }
});