var app = angular.module('PasswordMonitor');
app.factory('MonitorService', function ($q, $window, IdbService, TimerService, ManglingService, HashingService) {

    chrome.webRequest.onBeforeRequest.addListener(function (dict) {
        if (dict && dict.tabId) {
            chrome.tabs.get(dict.tabId, function (tab) {
                if (tab && tab.title) {
                    IdbService.insertNewOAuthEntry(dict.url, tab.url, new Date().getTime());
                }
            });
        }
    }, {
        urls: [
            "https://graph.facebook.com/*",
            "https://accounts.google.com/o/oauth2/auth*"
        ]
    });

    function getStructure(password) {
        var charArray = password.split("");
        var structure = "";
        for (var i = 0; i < charArray.length; i++) {
            if (charArray[i].match(/[A-Z]/)) {
                structure += "U";
            }
            else if (charArray[i].match(/[a-z]/)) {
                structure += "L";
            }
            else if (charArray[i].match(/\d/)) {
                structure += "D";
            }
            else {
                structure += "S";
            }
        }
        return structure;
    }

    function getNumCharClasses(password) {

        var numCharClasses = 0;
        angular.forEach([/[A-Z]/, /[a-z]/, /\d/, /[!"#\$%&'()\*\+,-\./:;<=>\?@\[\\\]\^_`{|}~]/], function (expr) {

            if (password.match(expr)) {

                numCharClasses++;
            }
        });
        return numCharClasses
    }

    function mangleAndHash(password) {
        var qs = [];
        var variations = ManglingService.mangle(password);
        console.log("There are " + variations.length + " variations.");
        for (var i = 0; i < variations.length; i++) {
            (function (i) {
                var deferred = $q.defer();
                HashingService.hashThis(variations[i]['variation']).then(function (result) {
                    deferred.resolve({
                        variationHash: result,
                        reason: variations[i]['reason']
                    });
                });
                qs.push(deferred.promise);
            })(i);
        }
        return $q.all(qs);
    }

    return {

        newPassword: function (url, username, password, time, timeSpent) {

            if (url && username && password && time && timeSpent && !TimerService.isSuspended()) {
                chrome.storage.local.get('exclusionList', function (result) {
                    if (result['exclusionList']) {
                        for (var i = 0; i < result['exclusionList'].length; i++) {
                            if (url.match(result['exclusionList'][i])) {
                                return;
                            }
                        }
                    }
                    var pwdHash, unHash;
                    $q.all({
                        username: HashingService.hashThis(username),
                        password: HashingService.hashThis(password)
                    }).then(function (results) {
                        pwdHash = results.password;
                        unHash = (username === 'unknown') ? username : results.username;
                        IdbService.insertNewUsageEntry(unHash, pwdHash, url, time, timeSpent);
                        return IdbService.hasPasswordAlready(pwdHash);
                    }).then(function (hasPasswordAlready) {
                        if (hasPasswordAlready) {
                            return $q.reject();
                        }
                        console.time("hash");
                        return mangleAndHash(password);
                    }).then(function (variations) {
                        console.timeEnd("hash");
                        var structure = getStructure(password);
                        var length = password.length;
                        var numCharClasses = getNumCharClasses(password);
                        var zxcvbn = $window.zxcvbn(password);
                        IdbService.insertNewPasswordEntry(unHash, pwdHash, url, time, timeSpent, length, numCharClasses,
                            structure, zxcvbn.entropy, zxcvbn.crack_time_display, variations);
                    });
                });
            }
        }
    };
});