var app = angular.module('PasswordMonitor');
/**
 * This filter takes all the credential uses, and filters them by each password
 */
app.filter('pwdFilter', function () {
    return function (credentialUses, passwordHash) {
        if (credentialUses) {
            var filtered = [];
            for (var n = 0; n < credentialUses.length; n++) {
                if (credentialUses[n]['passwordHash'] === passwordHash) {
                    filtered.push(credentialUses[n]);
                }
            }
            return filtered;
        }
        else {
            return [];
        }
    };
});
app.controller('SettingsCtrl', function ($scope, $modal, $filter, TimerService) {

    /**
     * This opens a modal if the user has not entered a salt yet, as this is required for the extension to work
     */
    chrome.runtime.sendMessage({'type': 'getSalt'}, function (salt) {
        if (!salt) {
            $modal.open({
                templateUrl: chrome.runtime.getURL('html/templates/EmailModal.html'),
                controller: 'EmailModalCtrl',
                backdrop: 'static'
            });
        }
    });

    /**
     * Function for uploading data
     */
    $scope.button = {
        uploading: false,
        text: "Upload Data"
    };
    $scope.uploadData = function () {
        $scope.button.uploading = true;
        $scope.button.text = "Uploading...";
        var modalInstance = $modal.open({
            templateUrl: chrome.runtime.getURL('html/templates/DataUploadModal.html'),
            controller: 'DataUploadModalCtrl',
            backdrop: 'static'
        });
        modalInstance.result.then(function () {
            $scope.button.uploading = false;
            $scope.button.text = "Upload Data";
        });
    };

    /**
     * Timer Controls
     */
    $scope.time = {
        options: {
            size: 150
        },
        isSuspended: TimerService.isSuspended,
        minutes: TimerService.getMinutes,
        seconds: TimerService.getSeconds,
        percent: TimerService.getPercent
    };
    chrome.runtime.sendMessage({'type': 'getTime'}, function (time) {
        if (time) {
            TimerService.startTimer(time);
        }
    });
    $scope.startTimer = function (seconds) {
        var time = {
            startTime: seconds === undefined ? 0 : seconds,
            totalSeconds: seconds === undefined ? 0 : seconds
        };
        chrome.runtime.sendMessage({'type': 'startTimer', 'time': time});
    };
    $scope.stopTimer = function () {
        chrome.runtime.sendMessage({'type': 'stopTimer'});
    };
    $scope.alerts = [];
    $scope.startCustomTimer = function (custom) {
        if (this.customTime.$valid) {
            var sec = ((custom['hr'] * 60) + custom['min']) * 60;
            $scope.startTimer(sec);
        }
        else {
            $scope.alerts.push('Cannot suspend for more than 11:59');
        }
    };
    $scope.closeAlert = function (index) {
        $scope.alerts.splice(index, 1);
    };

    /**
     * Email controls
     */
    chrome.storage.local.get('email', function (result) {
        if (result['email'])
          $scope.email = result['email'];
        else
          $scope.email = '';
    });
    $scope.setEmailWithoutChangingHash = function (mail) {
        if (mail) {
            chrome.runtime.sendMessage({type: 'setEmailNH', email: mail});
        }
    };

    /**
     * Concurrency Level controls
     */
    chrome.runtime.sendMessage({type: 'getConcurrency'}, function (concurrency) {
        $scope.concurrency = concurrency;
    });
    $scope.setConcurrency = function (number) {
        chrome.runtime.sendMessage({type: 'setConcurrency', numberOfWorkers: number});
    };

    /**
     * Controls for the Exclusion List
     */
    $scope.exclusionList = [];
    $scope.history = [];
    ($scope.refreshExclusionList = function () {
        chrome.storage.local.get('exclusionList', function (result) {
            if (angular.isDefined(result['exclusionList'])) {
                $scope.$apply($scope.exclusionList = result['exclusionList']);
            }
        });
    })();
    chrome.history.search({text: "", maxResults: 500}, function (results) {
        angular.forEach(results, function (result) {
            var canonicalUrl = result['url'].split("/")[2];
            if ($scope.history.indexOf(canonicalUrl) === -1) {
                $scope.history.push(canonicalUrl);
            }
        });
    });
    $scope.addToExclusionList = function (item) {
        if (item) {
            $scope.exclusionList.push(angular.copy(item));
            chrome.storage.local.set({'exclusionList': $scope.exclusionList});
            $scope.newExclusion = "";
        }
    };
    $scope.removeFromExclusionList = function (index) {
        if (index >= 0) {
            $scope.exclusionList.splice(index, 1);
            chrome.storage.local.set({'exclusionList': $scope.exclusionList});
        }
    };

    /**
     * Controls for the list of captured hashes
     */
    $scope.passwordMetrics = [];
    chrome.runtime.sendMessage({'type': 'getPasswordMetrics'}, function (result) {
        $scope.$apply($scope.passwordMetrics = result.passwordMetrics);
    });

    $scope.credentialUses = [];
    chrome.runtime.sendMessage({'type': 'getCredentialUses'}, function (result) {
        $scope.$apply($scope.credentialUses = result.credentialUses);
    });
    $scope.toggleMasterCheck = function (passwordHash, checked) {
        var filtered = $filter('pwdFilter')($scope.credentialUses, passwordHash);
        for (var i = 0; i < filtered.length; i++) {
            filtered[i].checked = checked;
        }
    };
    $scope.anyChecked = function () {
        var anyChecked = false;
        angular.forEach($scope.credentialUses, function (use) {
            anyChecked |= use.checked;
        });
        return anyChecked;
    };
    $scope.refresh = function () {

        // Make a note if any metrics item is expanded, so we can keep it expanded after refreshing
        var open = [];
        angular.forEach($scope.passwordMetrics, function (password) {
            if (password.open) {
                open.push(password.passwordHash);
            }
        });

        chrome.runtime.sendMessage({'type': 'getPasswordMetrics'}, function (result) {
            $scope.$apply(function () {
                $scope.passwordMetrics = result.passwordMetrics;
                // Set the same password as expanded again
                angular.forEach($scope.passwordMetrics, function (password) {
                    if (open.indexOf(password.passwordHash) > -1) {
                        password.open = true;
                    }
                });
            });
        });
        chrome.runtime.sendMessage({'type': 'getCredentialUses'}, function (result) {
            $scope.$apply($scope.credentialUses = result.credentialUses);
        });
    };
    $scope.deleteEntries = function () {
        chrome.runtime.sendMessage({
            'type': 'deleteCheckedEntries',
            'credentialUses': $scope.credentialUses
        }, function () {
            $scope.refresh();
        });
    };

    $scope.date = new Date();

    /**
     * Controls for Hashing Tool
     */
    $scope.clear = function () {
        for (var i = 0; i < $scope.passwordMetrics.length; i++) {
            (function (i) {
                $scope.passwordMetrics[i].matched = false;
            })(i);
        }
    };
    $scope.hash = function (cleartext) {
        if (cleartext) {
            $scope.hashing = true;
            $scope.clear();
            chrome.runtime.sendMessage({'type': 'hashThis', 'cleartext': cleartext}, function (hash) {
                $scope.$apply(function () {
                    $scope.hashResult = hash;
                    $scope.hashing = false;
                });
                for (var i = 0; i < $scope.passwordMetrics.length; i++) {
                    (function (i) {
                        if ($scope.passwordMetrics[i].passwordHash === hash) {
                            $scope.$apply($scope.passwordMetrics[i].matched = true);
                        }
                    })(i);
                }
            });
        }
    };

    /**
     * Open Chrome Extensions in a new tab
     */
    $scope.openChromeExtensions = function () {
        chrome.runtime.sendMessage({'type': 'openChromeExtensions'});
    };

    /**
     * React to incoming messages
     */
    chrome.runtime.onMessage.addListener(function (message) {

        switch (message['type']) {

            case 'upload':
                console.log(message['value']);
                break;

            case 'startTimer':
                if (message['time']) {
                    TimerService.startTimer(message['time']);
                }
                break;

            case 'stopTimer':
                TimerService.stopTimer();
                break;
        }
    })
});
