angular.module('PasswordMonitor').factory('TimerService', function ($interval) {

    var time;

    function stopTimer() {
        $interval.cancel(time.timer);
        time = undefined;
    }

    return {

        isSuspended: function () {
            return angular.isDefined(time);
        },

        startTimer: function (incomingTime) {
            if (angular.isDefined(time)) {
                stopTimer();
            }
            time = incomingTime;
            time.timer = $interval(function () {
                if (time.totalSeconds > 0) {
                    time.totalSeconds--;
                }
                else {
                    stopTimer();
                }
            }, 1000);
        },

        stopTimer: stopTimer,

        getMinutes: function () {
            if (angular.isDefined(time)) {
                return Math.floor(time.totalSeconds / 60);
            }
            else {
                return null;
            }
        },

        getSeconds: function () {
            if (angular.isDefined(time)) {
                return time.totalSeconds % 60;
            }
            else {
                return null;
            }
        },

        getPercent: function () {
            if (angular.isDefined(time)) {
                return time.totalSeconds / time.startTime * 100;
            }
            else {
                return 0;
            }
        },

        getAll: function () {
            if (angular.isDefined(time)) {
                return {
                    totalSeconds: time.totalSeconds,
                    startTime: time.startTime
                };
            }
            else {
                return false;
            }
        }
    }
});