/**
 * Created by Nikolai on 02/04/15.
 */
describe('BackgroundCtrl', function () {

    var ctrl, scope, hashingService, dataUploadService, monitorService, idbService, timerService, iconService;
    var onInstalledCallback, onConnectCallback, onMessageCallback, tabCreateConfig;
    beforeEach(function () {
        module('PasswordMonitor', function ($provide) {
            var stub = {
                addListener: function () {
                }
            };
            $provide.decorator('$window', function ($delegate) {
                $delegate.chrome = {
                    runtime: {
                        onInstalled: {
                            addListener: function (callback) {
                                onInstalledCallback = callback;
                            }
                        },
                        onConnect: {
                            addListener: function (callback) {
                                onConnectCallback = callback;
                            }
                        },
                        onMessage: {
                            addListener: function (callback) {
                                onMessageCallback = callback;
                            }
                        }
                    },
                    storage: {
                        local: {
                            get: function () {
                            },
                            set: function () {
                            }
                        }
                    },
                    webRequest: {onBeforeRequest: stub},
                    tabs: {
                        onUpdated: stub,
                        onActivated: stub,
                        create: function (config) {
                            tabCreateConfig = config;
                        }
                    },
                    extension: {
                        getURL: function () {
                            return 'something.html';
                        }
                    }
                };
                return $delegate;
            })
        });
        inject(function ($controller, $rootScope, $window,
                         HashingService, DataUploadService, MonitorService, IdbService, TimerService, IconService) {
            scope = $rootScope.$new();
            hashingService = HashingService;
            dataUploadService = DataUploadService;
            monitorService = MonitorService;
            idbService = IdbService;
            timerService = TimerService;
            iconService = IconService;
            ctrl = $controller('BackgroundCtrl', {
                $scope: scope
            });
        });
    });

    afterEach(function () {
        tabCreateConfig = undefined;
    });

    it('should register all callbacks', function () {
        expect(onInstalledCallback).toBeDefined();
        expect(onConnectCallback).toBeDefined();
        expect(onMessageCallback).toBeDefined();
    });

    describe('onInstalled', function () {
        it('should call HashingService.getSalt if callback returns', function () {
            spyOn(hashingService, 'getSalt').and.callThrough();
            onInstalledCallback();
            expect(hashingService.getSalt).toHaveBeenCalled();
        });

        it('should open tab if callback returns false', function () {
            spyOn(hashingService, 'getSalt').and.returnValue({
                then: function (callback) {
                    callback(false);
                }
            });
            onInstalledCallback();
            expect(tabCreateConfig).toEqual({url: 'something.html', active: true});
        });

        it('should not open tab if callback returns true', function () {
            spyOn(hashingService, 'getSalt').and.returnValue({
                then: function (callback) {
                    callback(true);
                }
            });
            onInstalledCallback();
            expect(tabCreateConfig).toBeUndefined();
        })
    });

    describe('onConnect', function () {
        it('should call DataUploadService.uploadData if callback returns with correct port', function () {
            spyOn(dataUploadService, 'uploadData');
            onConnectCallback({name: 'uploadData'});
            expect(dataUploadService.uploadData).toHaveBeenCalledWith({name: 'uploadData'});
        });

        it('should not call DataUploadService.uploadData if callback returns with incorrect port', function () {
            spyOn(dataUploadService, 'uploadData');
            onConnectCallback({name: 'wrong'});
            expect(dataUploadService.uploadData).not.toHaveBeenCalled();
        });

        it('should not call DataUploadService.uploadData is callback arg is empty', function () {
            spyOn(dataUploadService, 'uploadData');
            onConnectCallback({});
            expect(dataUploadService.uploadData).not.toHaveBeenCalled();
        });

        it('should not call DataUploadService.uploadData is callback arg is missing', function () {
            spyOn(dataUploadService, 'uploadData');
            onConnectCallback();
            expect(dataUploadService.uploadData).not.toHaveBeenCalled();
        });
    });

    describe('onMessage', function () {
        it('should not do anything if message is missing', function () {
            spyOn(idbService, 'deleteCheckedEntries');
            spyOn(idbService, 'deleteEntriesFromUrl');
            spyOn(idbService, 'getAllValues').and.callThrough();
            spyOn(idbService, 'hasCollectedFromThisSite').and.callThrough();
            spyOn(monitorService, 'newPassword');
            spyOn(timerService, 'startTimer');
            spyOn(timerService, 'stopTimer');
            spyOn(iconService, 'suspend');
            spyOn(iconService, 'resume');
            spyOn(hashingService, 'getSalt').and.callThrough();
            spyOn(hashingService, 'hashThis').and.callThrough();
            spyOn(window.chrome.tabs, 'create').and.callThrough();
            spyOn(window.chrome.storage.local, 'set');
            onMessageCallback();
            expect(idbService.deleteCheckedEntries).not.toHaveBeenCalled();
            expect(idbService.deleteEntriesFromUrl).not.toHaveBeenCalled();
            expect(idbService.getAllValues).not.toHaveBeenCalled();
            expect(idbService.hasCollectedFromThisSite).not.toHaveBeenCalled();
            expect(monitorService.newPassword).not.toHaveBeenCalled();
            expect(timerService.startTimer).not.toHaveBeenCalled();
            expect(timerService.stopTimer).not.toHaveBeenCalled();
            expect(iconService.suspend).not.toHaveBeenCalled();
            expect(iconService.resume).not.toHaveBeenCalled();
            expect(hashingService.getSalt).not.toHaveBeenCalled();
            expect(hashingService.hashThis).not.toHaveBeenCalled();
            expect(window.chrome.tabs.create).not.toHaveBeenCalled();
            expect(window.chrome.storage.local.set).not.toHaveBeenCalled();
        });

        it('should not do anything if message is empty', function () {
            spyOn(idbService, 'deleteCheckedEntries');
            spyOn(idbService, 'deleteEntriesFromUrl');
            spyOn(idbService, 'getAllValues').and.callThrough();
            spyOn(idbService, 'hasCollectedFromThisSite').and.callThrough();
            spyOn(monitorService, 'newPassword');
            spyOn(timerService, 'startTimer');
            spyOn(timerService, 'stopTimer');
            spyOn(iconService, 'suspend');
            spyOn(iconService, 'resume');
            spyOn(hashingService, 'getSalt').and.callThrough();
            spyOn(hashingService, 'hashThis').and.callThrough();
            spyOn(window.chrome.tabs, 'create').and.callThrough();
            spyOn(window.chrome.storage.local, 'set');
            onMessageCallback({});
            expect(idbService.deleteCheckedEntries).not.toHaveBeenCalled();
            expect(idbService.deleteEntriesFromUrl).not.toHaveBeenCalled();
            expect(idbService.getAllValues).not.toHaveBeenCalled();
            expect(idbService.hasCollectedFromThisSite).not.toHaveBeenCalled();
            expect(monitorService.newPassword).not.toHaveBeenCalled();
            expect(timerService.startTimer).not.toHaveBeenCalled();
            expect(timerService.stopTimer).not.toHaveBeenCalled();
            expect(iconService.suspend).not.toHaveBeenCalled();
            expect(iconService.resume).not.toHaveBeenCalled();
            expect(hashingService.getSalt).not.toHaveBeenCalled();
            expect(hashingService.hashThis).not.toHaveBeenCalled();
            expect(window.chrome.tabs.create).not.toHaveBeenCalled();
            expect(window.chrome.storage.local.set).not.toHaveBeenCalled();
        });

        it('should call MonitorService.newPassword if type is newPassword', function () {
            spyOn(monitorService, 'newPassword');
            onMessageCallback({
                type: 'newPassword',
                url: 'my.website.com',
                username: '@johnJohnson',
                password: 'password123',
                time: 1234,
                timeSpent: 56789
            });
            expect(monitorService.newPassword).toHaveBeenCalledWith(
                'my.website.com',
                '@johnJohnson',
                'password123',
                1234,
                56789
            );
        });

        it('should not call MonitorService.newPassword if any field is missing', function () {
            spyOn(monitorService, 'newPassword');
            onMessageCallback({
                type: 'newPassword',
                url: undefined,
                username: '@johnJohnson',
                password: 'password123',
                time: 1234,
                timeSpent: 56789
            });
            expect(monitorService.newPassword).not.toHaveBeenCalled();
            onMessageCallback({
                type: 'newPassword',
                url: 'my.website.com',
                username: '',
                password: 'password123',
                time: 1234,
                timeSpent: 56789
            });
            expect(monitorService.newPassword).not.toHaveBeenCalled();
            onMessageCallback({
                type: 'newPassword',
                url: 'my.website.com',
                username: '@johnJohnson',
                password: undefined,
                time: 1234,
                timeSpent: 56789
            });
            expect(monitorService.newPassword).not.toHaveBeenCalled();
            onMessageCallback({
                type: 'newPassword',
                url: 'my.website.com',
                username: '@johnJohnson',
                password: 'password123',
                time: 0,
                timeSpent: 56789
            });
            expect(monitorService.newPassword).not.toHaveBeenCalled();
            onMessageCallback({
                type: 'newPassword',
                url: 'my.website.com',
                username: '@johnJohnson',
                password: 'password123',
                time: 1234,
                timeSpent: 0
            });
            expect(monitorService.newPassword).not.toHaveBeenCalled();
        });

        it('should call IdbService.getAllValues when type is getPasswordMetrics', function () {
            spyOn(idbService, 'getAllValues').and.callThrough();
            onMessageCallback({type: 'getPasswordMetrics'});
            expect(idbService.getAllValues).toHaveBeenCalledWith('metrics');
        });

        it('should return true when type is getPasswordMetrics', function () {
            expect(onMessageCallback({type: 'getPasswordMetrics'})).toBeTruthy();
        });

        it('should call callback when IdbService.getAllValues returns', function () {
            spyOn(idbService, 'getAllValues').and.returnValue({
                then: function (callback) {
                    callback('result');
                }
            });
            var callback = jasmine.createSpy('callback');
            onMessageCallback({type: 'getPasswordMetrics'}, 'sender', callback);
            expect(callback).toHaveBeenCalledWith({passwordMetrics: 'result'});
        });

        it('should call IdbService.getAllValues when type is getCredentialUses', function () {
            spyOn(idbService, 'getAllValues').and.callThrough();
            onMessageCallback({type: 'getCredentialUses'});
            expect(idbService.getAllValues).toHaveBeenCalledWith('credentialUses');
        });

        it('should return true when type is getCredentialUses', function () {
            expect(onMessageCallback({type: 'getCredentialUses'})).toBeTruthy();
        });

        it('should call callback when IdbService.getAllValues returns', function () {
            spyOn(idbService, 'getAllValues').and.returnValue({
                then: function (callback) {
                    callback('result');
                }
            });
            var callback = jasmine.createSpy('callback');
            onMessageCallback({type: 'getCredentialUses'}, 'sender', callback);
            expect(callback).toHaveBeenCalledWith({credentialUses: 'result'});
        });

        it('should call IdbService.deleteCheckedEntries when type is deleteCheckedEntries', function () {
            spyOn(idbService, 'deleteCheckedEntries');
            var callback = function () {
            };
            onMessageCallback({type: 'deleteCheckedEntries', credentialUses: 'some uses'}, 'me', callback);
            expect(idbService.deleteCheckedEntries).toHaveBeenCalledWith('some uses', callback);
        });

        it('should return true when type is deleteCheckedEntries', function () {
            expect(onMessageCallback({type: 'deleteCheckedEntries'})).toBeTruthy();
        });

        it('should call IdbService.deleteEntriesFromUrl when type is deleteEntriesFromUrl', function () {
            spyOn(idbService, 'deleteEntriesFromUrl');
            onMessageCallback({type: 'deleteEntriesFromUrl', url: 'some URL'});
            expect(idbService.deleteEntriesFromUrl).toHaveBeenCalledWith('some URL');
        });

        it('should not call IdbService.deleteEntriesFromUrl if url is empty or missing', function () {
            spyOn(idbService, 'deleteEntriesFromUrl');
            onMessageCallback({type: 'deleteEntriesFromUrl', url: ''});
            expect(idbService.deleteEntriesFromUrl).not.toHaveBeenCalled();
            onMessageCallback({type: 'deleteEntriesFromUrl'});
            expect(idbService.deleteEntriesFromUrl).not.toHaveBeenCalled();
        });

        it('should call TimerService.startTimer and IconService.suspend when type is startTimer', function () {
            spyOn(timerService, 'startTimer');
            spyOn(iconService, 'suspend');
            onMessageCallback({type: 'startTimer', time: 1234});
            expect(timerService.startTimer).toHaveBeenCalledWith(1234);
            expect(iconService.suspend).toHaveBeenCalled();
        });

        it('should not call TimerService.startTimer if time is 0 or missing', function () {
            spyOn(timerService, 'startTimer');
            spyOn(iconService, 'suspend');
            onMessageCallback({type: 'startTimer', time: 0});
            expect(timerService.startTimer).not.toHaveBeenCalled();
            expect(iconService.suspend).not.toHaveBeenCalled();
            onMessageCallback({type: 'startTimer'});
            expect(timerService.startTimer).not.toHaveBeenCalled();
            expect(iconService.suspend).not.toHaveBeenCalled();
        });

        it('should call TimerService.stopTimer and IconService.resume when type is stopTimer', function () {
            spyOn(timerService, 'stopTimer');
            spyOn(iconService, 'resume');
            onMessageCallback({type: 'stopTimer'});
            expect(timerService.stopTimer).toHaveBeenCalled();
            expect(iconService.resume).toHaveBeenCalled();
        });

        it('should call callback with results from TimerService.getAll when type is getTime', function () {
            var callback = jasmine.createSpy('callback');
            onMessageCallback({type: 'getTime'}, 'me', callback);
            expect(callback).toHaveBeenCalledWith(timerService.getAll());
        });

        it('should return true when type is getTime', function () {
            expect(onMessageCallback({type: 'getTime'}, 'some user', function () {
            })).toBeTruthy();
        });

        it('should open new tab when type is openChromeExtensions', function () {
            spyOn(window.chrome.tabs, 'create').and.callThrough();
            onMessageCallback({type: 'openChromeExtensions'});
            expect(window.chrome.tabs.create).toHaveBeenCalledWith({url: "chrome://extensions/"});
            expect(tabCreateConfig).toEqual({url: "chrome://extensions/"});
        });

        it('should call HashingService.getSalt when type is getSalt', function () {
            spyOn(hashingService, 'getSalt').and.callThrough();
            onMessageCallback({type: 'getSalt'});
            expect(hashingService.getSalt).toHaveBeenCalled();
        });

        it('should call callback with salt when it returns when type is getSalt', function () {
            spyOn(hashingService, 'getSalt').and.returnValue({
                then: function (callback) {
                    callback('salt');
                }
            });
            var callback = jasmine.createSpy('callback');
            onMessageCallback({type: 'getSalt'}, 'some user', callback);
            expect(callback).toHaveBeenCalledWith('salt');
        });

        it('should return true when type is getSalt', function () {
            expect(onMessageCallback({type: 'getSalt'})).toBeTruthy();
        });

        it('should call HashingService.hashThis when type is setEmail', function () {
            spyOn(hashingService, 'hashThis').and.callThrough();
            onMessageCallback({type: 'setEmail', email: 'email@email.com'});
            expect(hashingService.hashThis).toHaveBeenCalledWith('email@email.com');
        });

        it('should set email in chrome storage when hash returns when type is setEmail', function () {
            spyOn(hashingService, 'hashThis').and.returnValue({
                then: function (callback) {
                    callback('hashed');
                }
            });
            spyOn(window.chrome.storage.local, 'set');
            onMessageCallback({type: 'setEmail', email: 'email@email.com'});
            expect(window.chrome.storage.local.set).toHaveBeenCalledWith({
                'email': 'email@email.com',
                'salt': 'hashed'
            });
        });

        it('should not call HashingService.hashThis if email is empty or missing', function () {
            spyOn(hashingService, 'hashThis');
            onMessageCallback({type: 'setEmail', email: ''});
            expect(hashingService.hashThis).not.toHaveBeenCalled();
            onMessageCallback({type: 'setEmail'});
            expect(hashingService.hashThis).not.toHaveBeenCalled();
        });

        it('should call IdbService.hasCollectedFromThisSite when type is hasCollectedFromThisSite', function () {
            spyOn(idbService, 'hasCollectedFromThisSite').and.callThrough();
            onMessageCallback({type: 'hasCollectedFromThisSite', url: 'some.site.com'});
            expect(idbService.hasCollectedFromThisSite).toHaveBeenCalledWith('some.site.com');
        });

        it('should return true when type is hasCollectedFromThisSite', function () {
            expect(onMessageCallback({type: 'hasCollectedFromThisSite'})).toBeTruthy();
        });

        it('should call callback with boolean when type is hasCollectedFromThisSite', function () {
            spyOn(idbService, 'hasCollectedFromThisSite').and.returnValue({
                then: function (callback) {
                    callback(true);
                }
            });
            var callback = jasmine.createSpy('callback');
            onMessageCallback({type: 'hasCollectedFromThisSite', url: 'some.web.site'}, 'some user', callback);
            expect(callback).toHaveBeenCalledWith(true);
        });

        it('should not call IdbService.hasCollectedFromThisSite if URL is empty or missing', function () {
            spyOn(idbService, 'hasCollectedFromThisSite').and.callThrough();
            onMessageCallback({type: 'hasCollectedFromThisSite', url: ''});
            expect(idbService.hasCollectedFromThisSite).not.toHaveBeenCalled();
            onMessageCallback({type: 'hasCollectedFromThisSite'});
            expect(idbService.hasCollectedFromThisSite).not.toHaveBeenCalled();
        });

        it('should call HashingService.hashThis when type is hashThis', function () {
            spyOn(hashingService, 'hashThis').and.callThrough();
            onMessageCallback({type: 'hashThis', cleartext: 'some text'});
            expect(hashingService.hashThis).toHaveBeenCalledWith('some text');
        });

        it('should return true when type is hashThis', function () {
            expect(onMessageCallback({type: 'hashThis', cleartext: 'some text'})).toBeTruthy();
        });

        it('should call callback with result when type is hashThis', function () {
            spyOn(hashingService, 'hashThis').and.returnValue({
                then: function (callback) {
                    callback('hashed');
                }
            });
            var callback = jasmine.createSpy('callback');
            onMessageCallback({type: 'hashThis', cleartext: 'some text'}, 'some user', callback);
            expect(callback).toHaveBeenCalledWith('hashed');
        });

        it('should not call HashingService.hashThis if cleartext is empty or missing', function () {
            spyOn(hashingService, 'hashThis').and.callThrough();
            onMessageCallback({type: 'hashThis', cleartext: ''});
            expect(hashingService.hashThis).not.toHaveBeenCalled();
            onMessageCallback({type: 'hashThis'});
            expect(hashingService.hashThis).not.toHaveBeenCalled();
        })
    })
});