/**
 * Created by Nikolai on 04/04/15.
 */
describe('DataUploadService', function () {

    var service, idbService, http, q, scope, uploadURL, compile, xhr, error, midwayError;
    beforeEach(function () {
        module('PasswordMonitor', function ($provide) {
            $provide.decorator('$window', function ($delegate) {
                $delegate.chrome = {
                    storage: {
                        local: {
                            get: function (email, callback) {
                                if (email === 'email') {
                                    callback({email: 'email@email.com'});
                                }
                            }
                        }
                    }
                };
                $delegate.XMLHttpRequest = (function () {
                    xhr = this;
                    this.open = function (metod, url) {
                        this.method = metod;
                        this.url = url;
                    };
                    this.upload = {};
                    this.response = {
                        DirectUpload: error ? 'Error' : 'Success'
                    };
                    this.send = function (blob) {
                        this.response.DataLength = blob.size;
                        this.upload.onprogress({loaded: 25, total: 100});
                        if (midwayError) {
                            this.onerror({code: 123, statusText: 'test error'});
                        }
                        else {
                            this.upload.onprogress({loaded: 50, total: 100});
                            this.upload.onprogress({loaded: 75, total: 100});
                            this.onload();
                        }
                    };
                });
                return $delegate;
            })
        });
        inject(function (DataUploadService, IdbService, $httpBackend, $q, $rootScope, $compile, serverRootUrl) {
            service = DataUploadService;
            idbService = IdbService;
            http = $httpBackend;
            q = $q;
            scope = $rootScope.$new();
            compile = $compile;
            uploadURL = serverRootUrl;
        })
    });

    afterEach(function () {
        xhr = undefined;
        error = undefined;
        midwayError = undefined;
    });

    it('should be defined', function () {
        expect(service).toBeDefined();
    });

    it('should have exposed function', function () {
        expect(service.uploadData).toBeDefined();
    });

    describe('uploadData', function () {
        it('should post message to port', function () {
            var port = jasmine.createSpyObj('port', ['postMessage']);
            service.uploadData(port);
            expect(port.postMessage).toHaveBeenCalledWith({
                inProgress: true,
                stage: 1,
                text: 'Retrieving data from tables'
            });
        });

        it('should call IdbService.getAllTables', function () {
            var port = jasmine.createSpyObj('port', ['postMessage']);
            spyOn(idbService, 'getAllTables').and.callThrough();
            service.uploadData(port);
            expect(idbService.getAllTables).toHaveBeenCalled();
        });

        it('should not continue if port is missing', function () {
            spyOn(idbService, 'getAllTables').and.callThrough();
            service.uploadData();
            expect(idbService.getAllTables).not.toHaveBeenCalled();
        });

        it('should not continue if postMessage function is missing', function () {
            spyOn(idbService, 'getAllTables').and.callThrough();
            service.uploadData({});
            expect(idbService.getAllTables).not.toHaveBeenCalled();
        });

        it('should post new message when IdbService returns tables', function () {
            var port = jasmine.createSpyObj('port', ['postMessage']);
            spyOn(idbService, 'getAllTables').and.returnValue({
                then: function (callback) {
                    callback({some: 'tables'});
                    return {
                        then: function () {
                            return {
                                then: function () {
                                    return {
                                        catch: function () {
                                        }
                                    }
                                }
                            }
                        }
                    };
                }
            });
            service.uploadData(port);
            expect(port.postMessage.calls.mostRecent().args).toEqual([{
                inProgress: true,
                stage: 2,
                text: 'Getting login status'
            }]);
        });

        it('should http GET if tables return', function () {
            var port = jasmine.createSpyObj('port', ['postMessage']);
            spyOn(idbService, 'getAllTables').and.returnValue({
                then: function (callback) {
                    callback({some: 'tables'});
                    return {
                        then: function () {
                            return {
                                then: function () {
                                    return {
                                        catch: function () {
                                        }
                                    }
                                }
                            }
                        }
                    };
                }
            });
            http.expectGET(uploadURL + 'a/login').respond(200);
            service.uploadData(port);
            http.flush();
        });

        it('should post yet another message to the port if first http returns successfully', function () {
            var port = jasmine.createSpyObj('port', ['postMessage', 'disconnect']);
            spyOn(idbService, 'getAllTables').and.returnValue({
                then: function (callback) {
                    var deferred = q.defer();
                    callback({some: 'tables'}).then(function (result) {
                        deferred.resolve(result);
                    });
                    return deferred.promise;
                }
            });
            http.expectGET(uploadURL + 'a/login')
                .respond(200, "<input type=\"hidden\" name=\"_csrf_token\" value=\"SOME_TOKEN\">");
            http.expectPOST(uploadURL + 'a/login_check').respond(200);
            service.uploadData(port);
            http.flush();
            expect(port.postMessage.calls.argsFor(2)).toEqual([{
                inProgress: true,
                stage: 3,
                text: 'Logging in as: email@email.com'
            }]);
        });

        // TODO: Få denne til å funke
        xit('should do http POST with the CSRF token, username and password', function () {
            var port = jasmine.createSpyObj('port', ['postMessage', 'disconnect']);
            spyOn(idbService, 'getAllTables').and.returnValue({
                then: function (callback) {
                    var deferred = q.defer();
                    callback({some: 'tables'}).then(function (result) {
                        deferred.resolve(result);
                    });
                    return deferred.promise;
                }
            });
            http.expectGET(uploadURL + 'a/login').respond(200, '<input name="_csrf_token" value="SOME_TOKEN">');
            http.expectPOST(
                uploadURL + 'a/login_check',
                '_username=nik%40nik.com&_password=nik%40nik.com&_remember_me=on&_csrf_token=undefined',
                {
                    'Content-Type': 'application/x-www-form-urlencoded',
                    "Accept": "application/json, text/plain, */*"
                }
            ).respond(200);
            service.uploadData(port);
            http.flush();
        });

        it('should post yet another message to the port if second http returns successfully', function () {
            var port = jasmine.createSpyObj('port', ['postMessage', 'disconnect']);
            spyOn(idbService, 'getAllTables').and.returnValue({
                then: function (callback) {
                    var deferred = q.defer();
                    callback({some: 'tables'}).then(function (result) {
                        deferred.resolve(result);
                    });
                    return deferred.promise;
                }
            });
            http.expectGET(uploadURL + 'a/login')
                .respond(200, "<input type=\"hidden\" name=\"_csrf_token\" value=\"SOME_TOKEN\">");
            http.expectPOST(uploadURL + 'a/login_check').respond(200);
            service.uploadData(port);
            http.flush();
            expect(port.postMessage.calls.argsFor(3)).toEqual([{inProgress: true, stage: 4, text: 'Sending data'}]);
        });

        it('should do http POST to upload data', function () {
            var port = jasmine.createSpyObj('port', ['postMessage', 'disconnect']);
            spyOn(idbService, 'getAllTables').and.returnValue({
                then: function (callback) {
                    var deferred = q.defer();
                    callback({some: 'tables'}).then(function (result) {
                        deferred.resolve(result);
                    });
                    return deferred.promise;
                }
            });
            http.expectGET(uploadURL + 'a/login').respond(200, '<input name="_csrf_token" value="SOME_TOKEN">');
            http.expectPOST(uploadURL + 'a/login_check').respond(200);
            service.uploadData(port);
            http.flush();
            expect(xhr.method).toEqual('POST');
            expect(xhr.url).toEqual(uploadURL + 'a/1/upload_direct');
        });

        it('should send 3 onprogress messages and a completion message', function () {
            var port = jasmine.createSpyObj('port', ['postMessage', 'disconnect']);
            spyOn(idbService, 'getAllTables').and.returnValue({
                then: function (callback) {
                    var deferred = q.defer();
                    callback({some: 'tables'}).then(function (result) {
                        deferred.resolve(result);
                    });
                    return deferred.promise;
                }
            });
            http.expectGET(uploadURL + 'a/login').respond(200, '<input name="_csrf_token" value="SOME_TOKEN">');
            http.expectPOST(uploadURL + 'a/login_check').respond(200);
            service.uploadData(port);
            http.flush();
            expect(port.postMessage.calls.argsFor(4)).toEqual([{inProgress: true, percentage: 25}]);
            expect(port.postMessage.calls.argsFor(5)).toEqual([{inProgress: true, percentage: 50}]);
            expect(port.postMessage.calls.argsFor(6)).toEqual([{inProgress: true, percentage: 75}]);
            expect(port.postMessage.calls.argsFor(7)).toEqual([{success: true}]);
        });

        it('should send error message if data length does not match', function () {
            error = true;
            var port = jasmine.createSpyObj('port', ['postMessage', 'disconnect']);
            spyOn(idbService, 'getAllTables').and.returnValue({
                then: function (callback) {
                    var deferred = q.defer();
                    callback({some: 'tables'}).then(function (result) {
                        deferred.resolve(result);
                    });
                    return deferred.promise;
                }
            });
            http.expectGET(uploadURL + 'a/login').respond(200, '<input name="_csrf_token" value="SOME_TOKEN">');
            http.expectPOST(uploadURL + 'a/login_check').respond(200);
            service.uploadData(port);
            http.flush();
            expect(port.postMessage.calls.argsFor(4)).toEqual([{inProgress: true, percentage: 25}]);
            expect(port.postMessage.calls.argsFor(5)).toEqual([{inProgress: true, percentage: 50}]);
            expect(port.postMessage.calls.argsFor(6)).toEqual([{inProgress: true, percentage: 75}]);
            expect(port.postMessage.calls.argsFor(7)).toEqual([{success: false}]);
        });

        it('should send error message if upload errors underway', function () {
            midwayError = true;
            var port = jasmine.createSpyObj('port', ['postMessage', 'disconnect']);
            spyOn(idbService, 'getAllTables').and.returnValue({
                then: function (callback) {
                    var deferred = q.defer();
                    callback({some: 'tables'}).then(function (result) {
                        deferred.resolve(result);
                    });
                    return deferred.promise;
                }
            });
            http.expectGET(uploadURL + 'a/login').respond(200, '<input name="_csrf_token" value="SOME_TOKEN">');
            http.expectPOST(uploadURL + 'a/login_check').respond(200);
            service.uploadData(port);
            http.flush();
            expect(port.postMessage.calls.argsFor(4)).toEqual([{inProgress: true, percentage: 25}]);
            expect(port.postMessage.calls.argsFor(5)).toEqual([{success: false, errorCode: 123, error: 'test error'}]);
        });

        it('should disconnect port when upload completed', function () {
            var port = jasmine.createSpyObj('port', ['postMessage', 'disconnect']);
            spyOn(idbService, 'getAllTables').and.returnValue({
                then: function (callback) {
                    var deferred = q.defer();
                    callback({some: 'tables'}).then(function (result) {
                        deferred.resolve(result);
                    });
                    return deferred.promise;
                }
            });
            http.expectGET(uploadURL + 'a/login').respond(200, '<input name="_csrf_token" value="SOME_TOKEN">');
            http.expectPOST(uploadURL + 'a/login_check').respond(200);
            service.uploadData(port);
            http.flush();
            expect(port.disconnect).toHaveBeenCalled();
        });

    })

});