/**
 * Created by Nikolai on 03/04/15.
 */
describe('IconService', function () {

    var service, onUpdatedCallback, onActivatedCallback;
    beforeEach(function () {
        module('PasswordMonitor', function ($provide) {
            $provide.decorator('$window', function ($delegate) {
                $delegate.chrome = {
                    browserAction: {
                        setBadgeBackgroundColor: function () {
                        },
                        setBadgeText: function () {
                        },
                        setIcon: function () {
                        }
                    },
                    storage: {
                        local: {
                            get: function () {
                            }
                        }
                    },
                    tabs: {
                        onUpdated: {
                            addListener: function (callback) {
                                onUpdatedCallback = callback;
                            }
                        },
                        onActivated: {
                            addListener: function (callback) {
                                onActivatedCallback = callback;
                            }
                        },
                        get: function () {
                        },
                        query: function () {
                        }
                    }
                };
                return $delegate;
            })
        });
        inject(function (IconService) {
            service = IconService;
        })
    });

    it('should exist', function () {
        expect(service).toBeDefined();
    });

    it('should expose all functions', function () {
        expect(service.suspend).toBeDefined();
        expect(service.resume).toBeDefined();
        expect(service.excluded).toBeDefined();
        expect(service.notExcluded).toBeDefined();
    });

    describe('suspend', function () {
        it('should call chrome.browserAction.setBadgeText', function () {
            spyOn(window.chrome.browserAction, 'setBadgeText');
            service.suspend();
            expect(window.chrome.browserAction.setBadgeText).toHaveBeenCalledWith({text: "OFF"});
        });

        it('should call chrome.browserAction.setBadgeBackgroundColor', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            service.suspend();
            expect(window.chrome.browserAction.setBadgeBackgroundColor).toHaveBeenCalledWith({color: "#F00"});
        });

        it('should call chrome.browserAction.setIcon', function () {
            spyOn(window.chrome.browserAction, 'setIcon');
            service.suspend();
            expect(window.chrome.browserAction.setIcon).toHaveBeenCalledWith({
                path: {
                    '19': '../image/icon19red.png',
                    '38': '../image/icon38red.png'
                }
            });
        });
    });

    describe('resume', function () {
        it('should call chrome.browserAction.setIcon', function () {
            spyOn(window.chrome.browserAction, 'setIcon');
            service.resume();
            expect(window.chrome.browserAction.setIcon).toHaveBeenCalledWith({
                path: {
                    '19': '../image/icon19.png',
                    '38': '../image/icon38.png'
                }
            });
        });

        it('should call chrome.tabs.query', function () {
            spyOn(window.chrome.tabs, 'query');
            service.resume();
            expect(window.chrome.tabs.query).toHaveBeenCalledWith({currentWindow: true, active: true},
                jasmine.any(Function));
        });

        it('should call chrome.storage.local.get when tabs query callback returns', function () {
            spyOn(window.chrome.storage.local, 'get');
            spyOn(window.chrome.tabs, 'query').and.callFake(function (query, callback) {
                callback([{url: 'some.url'}]);
            });
            service.resume();
            expect(window.chrome.storage.local.get).toHaveBeenCalledWith('exclusionList', jasmine.any(Function));
        });

        it('should not do anything if storage does not return exclusion list', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({});
            });
            spyOn(window.chrome.tabs, 'query').and.callFake(function (query, callback) {
                callback([{url: 'some.url'}]);
            });
            service.resume();
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).not.toHaveBeenCalled();
        });

        it('should not do anything if url is empty', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({});
            });
            spyOn(window.chrome.tabs, 'query').and.callFake(function (query, callback) {
                callback([{url: ''}]);
            });
            service.resume();
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).not.toHaveBeenCalled();
        });

        it('should not do anything if url is missing', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({});
            });
            spyOn(window.chrome.tabs, 'query').and.callFake(function (query, callback) {
                callback([{}]);
            });
            service.resume();
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).not.toHaveBeenCalled();
        });

        it('should not do anything if tabs query returns empty', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({});
            });
            spyOn(window.chrome.tabs, 'query').and.callFake(function (query, callback) {
                callback([]);
            });
            service.resume();
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).not.toHaveBeenCalled();
        });

        it('should remove badge if storage returns exclusion list and it does not contain url', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({exclusionList: []});
            });
            spyOn(window.chrome.tabs, 'query').and.callFake(function (query, callback) {
                callback([{url: 'some.url'}]);
            });
            service.resume();
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).toHaveBeenCalledWith({text: ""});
        });

        it('should set badge and background colour if storage returns exclusion list and it contains url', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({exclusionList: ['some.secure.url']});
            });
            spyOn(window.chrome.tabs, 'query').and.callFake(function (query, callback) {
                callback([{url: 'https://some.secure.url'}]);
            });
            service.resume();
            expect(window.chrome.browserAction.setBadgeBackgroundColor).toHaveBeenCalledWith({color: "#FF0"});
            expect(window.chrome.browserAction.setBadgeText).toHaveBeenCalledWith({text: "Stop"});
        });
    });

    describe('excluded', function () {
        it('should call setBadgeBackgroundColor and setBadgeText', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            service.excluded();
            expect(window.chrome.browserAction.setBadgeBackgroundColor).toHaveBeenCalledWith({color: "#FF0"});
            expect(window.chrome.browserAction.setBadgeText).toHaveBeenCalledWith({text: "Stop"});

        })
    });

    describe('notExcluded', function () {
        it('should call setBadgeText', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            service.notExcluded();
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).toHaveBeenCalledWith({text: ""});
        })
    });

    describe('onUpdated', function () {
        it('should set badge text and colour if tab is in exclusion list', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({exclusionList: ['some.secure.url']});
            });
            onUpdatedCallback(0, {url: 'https://some.secure.url'}, {currentWindow: true, active: true});
            expect(window.chrome.browserAction.setBadgeBackgroundColor).toHaveBeenCalledWith({color: "#FF0"});
            expect(window.chrome.browserAction.setBadgeText).toHaveBeenCalledWith({text: "Stop"});
        });

        it('should remove badge if it is not in exclusion list', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({exclusionList: ['some.other.url']});
            });
            onUpdatedCallback(0, {url: 'https://some.secure.url'}, {currentWindow: true, active: true});
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).toHaveBeenCalledWith({text: ""});
        });

        it('should not do anything if tab URL is empty', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({exclusionList: ['some.secure.url']});
            });
            onUpdatedCallback(0, {url: ''}, {currentWindow: true, active: true});
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).not.toHaveBeenCalled();
        });

        it('should not do anything if tab URL is missing', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({exclusionList: ['some.secure.url']});
            });
            onUpdatedCallback(0, {}, {currentWindow: true, active: true});
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).not.toHaveBeenCalled();
        });

        it('should not do anything if not current window', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({exclusionList: ['some.secure.url']});
            });
            onUpdatedCallback(0, {url: 'https://some.secure.url'}, {currentWindow: false, active: true});
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).not.toHaveBeenCalled();
        });

        it('should not do anything if not active tab', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({exclusionList: ['some.secure.url']});
            });
            onUpdatedCallback(0, {url: 'https://some.secure.url'}, {currentWindow: true, active: false});
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).not.toHaveBeenCalled();
        });

        it('should not do anything if suspended', function () {
            inject(function (TimerService) {
                spyOn(TimerService, 'isSuspended').and.returnValue(true);
                spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
                spyOn(window.chrome.browserAction, 'setBadgeText');
                spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                    callback({exclusionList: ['some.secure.url']});
                });
                onUpdatedCallback(0, {url: 'https://some.secure.url'}, {currentWindow: true, active: false});
                expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
                expect(window.chrome.browserAction.setBadgeText).not.toHaveBeenCalled();
            });
        })
    });

    describe('onActivated', function () {
        it('should not do anything if activeInfo is missing', function () {
            spyOn(window.chrome.tabs, 'get');
            onActivatedCallback();
            expect(window.chrome.tabs.get).not.toHaveBeenCalled();
        });

        it('should not do anything if tabId is empty', function () {
            spyOn(window.chrome.tabs, 'get');
            onActivatedCallback({tabId: ''});
            expect(window.chrome.tabs.get).not.toHaveBeenCalled();
        });

        it('should not do anything if tabId is missing', function () {
            spyOn(window.chrome.tabs, 'get');
            onActivatedCallback({});
            expect(window.chrome.tabs.get).not.toHaveBeenCalled();
        });

        it('should call chrome.tabs.get if tabId is provided', function () {
            spyOn(window.chrome.tabs, 'get');
            onActivatedCallback({tabId: 123});
            expect(window.chrome.tabs.get).toHaveBeenCalledWith(123, jasmine.any(Function));
        });

        it('should not do anything if tab is not found', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.tabs, 'get').and.callFake(function (tabId, callback) {
                expect(tabId).toBe(123);
                callback();
            });
            onActivatedCallback({tabId: 123});
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).not.toHaveBeenCalled();
        });

        it('should not do anything if tab URL is missing', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.tabs, 'get').and.callFake(function (tabId, callback) {
                expect(tabId).toBe(123);
                callback({});
            });
            onActivatedCallback({tabId: 123});
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).not.toHaveBeenCalled();
        });

        it('should not do anything if tab URL is empty', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.tabs, 'get').and.callFake(function (tabId, callback) {
                expect(tabId).toBe(123);
                callback({url: ''});
            });
            onActivatedCallback({tabId: 123});
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).not.toHaveBeenCalled();
        });

        it('should change badge and colour if tab is in exclusion list', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.tabs, 'get').and.callFake(function (tabId, callback) {
                expect(tabId).toBe(123);
                callback({url: 'https://some.secure.url'});
            });
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({exclusionList: ['some.secure.url']});
            });
            onActivatedCallback({tabId: 123});
            expect(window.chrome.browserAction.setBadgeBackgroundColor).toHaveBeenCalledWith({color: "#FF0"});
            expect(window.chrome.browserAction.setBadgeText).toHaveBeenCalledWith({text: "Stop"});
        });

        it('should change badge if tab is not in exclusion list', function () {
            spyOn(window.chrome.browserAction, 'setBadgeBackgroundColor');
            spyOn(window.chrome.browserAction, 'setBadgeText');
            spyOn(window.chrome.tabs, 'get').and.callFake(function (tabId, callback) {
                expect(tabId).toBe(123);
                callback({url: 'https://some.secure.url'});
            });
            spyOn(window.chrome.storage.local, 'get').and.callFake(function (query, callback) {
                callback({exclusionList: ['some.other.url']});
            });
            onActivatedCallback({tabId: 123});
            expect(window.chrome.browserAction.setBadgeBackgroundColor).not.toHaveBeenCalled();
            expect(window.chrome.browserAction.setBadgeText).toHaveBeenCalledWith({text: ""});
        })
    })
});
