/**
 * Created by Nikolai on 06/04/15.
 */
describe('IdbService', function () {

    var service, hashingService, q, dbRequest = [], trans, request, newEntry, deleteEntry, rootScope;
    var date = 1428325838915;
    beforeEach(function () {
        module('PasswordMonitor', function ($provide) {
            $provide.decorator('$window', function ($delegate) {
                $delegate.chrome = {
                    storage: {
                        local: {
                            get: function (email, callback) {
                                if (email === 'email') {
                                    callback({email: 'email@email.com'});
                                }
                            }
                        }
                    }
                };
                return $delegate;
            });
        });
        inject(function (IdbService, HashingService, $rootScope, $q) {
            service = IdbService;
            hashingService = HashingService;
            rootScope = $rootScope;
            q = $q;
        })
    });

    afterEach(function () {
        dbRequest = [];
        request = undefined;
        newEntry = undefined;
        deleteEntry = undefined;
    });

    beforeEach(function () {
        spyOn(hashingService, 'getSalt').and.returnValue('salt');
        spyOn(window.indexedDB, 'open').and.callFake(function () {
            var thisRequest = {
                result: {
                    objectStoreNames: ['metrics'],
                    transaction: function () {
                        return trans = {
                            objectStore: function (store) {
                                return {
                                    openCursor: function () {
                                        if (store === 'credentialUses') {
                                            return request = {
                                                result: {
                                                    value: {
                                                        url: 'some.other.url'
                                                    },
                                                    continue: function () {
                                                        if (this.value.url === 'some.other.url') {
                                                            this.value.url = 'some.url';
                                                            this.value.usernameHash = 'uh';
                                                            this.value.passwordHash = 'ph';
                                                            this.value.time = date;
                                                            request.onsuccess();
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                        else {
                                            return request = {
                                                result: {
                                                    value: 3,
                                                    continue: function () {
                                                        request.result.value--;
                                                        if (request.result.value === 0) {
                                                            request.result = undefined;
                                                        }
                                                        request.onsuccess();
                                                    }
                                                }
                                            };
                                        }
                                    },
                                    get: function (key) {
                                        if (typeof(key) === 'object') {
                                            return request = {
                                                result: {
                                                    url: 'some.other.url',
                                                    time: date
                                                }
                                            };
                                        }
                                        else {
                                            return request = {
                                                result: store === 'metrics' && key ? 'success' : undefined
                                            };
                                        }
                                    },
                                    add: function (entry) {
                                        if (newEntry === undefined) {
                                            newEntry = {};
                                        }
                                        if (store === 'passwordVariations') {
                                            if (newEntry.passwordVariations === undefined) {
                                                newEntry.passwordVariations = [entry];
                                            }
                                            else {
                                                newEntry.passwordVariations.push(entry);
                                            }
                                        }
                                        else {
                                            newEntry[store] = entry;
                                        }
                                        return request = {};
                                    },
                                    index: function () {
                                        return {
                                            openCursor: function () {
                                                return request = {
                                                    result: {
                                                        value: {
                                                            url: 'some.url'
                                                        },
                                                        primaryKey: 'previous retry',
                                                        continue: function () {
                                                            request.result = undefined;
                                                            request.onsuccess();
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    },
                                    delete: function (entry) {
                                        deleteEntry = entry;
                                        return request = {};
                                    }
                                };
                            }
                        };
                    }
                }
            };
            dbRequest.push(thisRequest);
            return thisRequest;
        });
        spyOn(hashingService, 'hashThis').and.callFake(function () {
            var deferred = q.defer();
            deferred.resolve('hashed');
            return deferred.promise;
        });
    });

    it("should be defined", function () {
        expect(service).toBeDefined();
    });

    it("should have exposed functions", function () {
        expect(service.getAllValues).toBeDefined();
        expect(service.hasPasswordAlready).toBeDefined();
        expect(service.insertNewPasswordEntry).toBeDefined();
        expect(service.insertNewUsageEntry).toBeDefined();
        expect(service.insertNewOAuthEntry).toBeDefined();
        expect(service.deleteCheckedEntries).toBeDefined();
        expect(service.deleteEntriesFromUrl).toBeDefined();
        expect(service.getAllTables).toBeDefined();
        expect(service.hasCollectedFromThisSite).toBeDefined();
        expect(service.getAllValues).toEqual(jasmine.any(Function));
        expect(service.hasPasswordAlready).toEqual(jasmine.any(Function));
        expect(service.insertNewPasswordEntry).toEqual(jasmine.any(Function));
        expect(service.insertNewUsageEntry).toEqual(jasmine.any(Function));
        expect(service.insertNewOAuthEntry).toEqual(jasmine.any(Function));
        expect(service.deleteCheckedEntries).toEqual(jasmine.any(Function));
        expect(service.deleteEntriesFromUrl).toEqual(jasmine.any(Function));
        expect(service.getAllTables).toEqual(jasmine.any(Function));
        expect(service.hasCollectedFromThisSite).toEqual(jasmine.any(Function));
    });

    describe('getAllValues', function () {
        it('should call get correct values', function (done) {
            service.getAllValues().then(function (result) {
                expect(result).toEqual([3, 2, 1]);
                done();
            });
            dbRequest[0].onsuccess();
            rootScope.$digest();
            request.onsuccess();
            rootScope.$digest();
        })
    });

    describe('hashPasswordAlready', function () {
        it("should return true if password hash already exist in DB", function (done) {
            service.hasPasswordAlready(true).then(function (result) {
                expect(result).toBeTruthy();
                done();
            });
            dbRequest[0].onsuccess();
            rootScope.$digest();
            request.onsuccess();
            rootScope.$digest();
        });

        it("should return false if password hash does not exist in DB", function (done) {
            service.hasPasswordAlready(false).then(function (result) {
                expect(result).toBeFalsy();
                done();
            });
            dbRequest[0].onsuccess();
            rootScope.$digest();
            request.onsuccess();
            rootScope.$digest();
        });
    });

    describe('insertNewPasswordEntry', function () {
        var date = new Date().getTime();
        it('should successfully put entry into metrics store', function () {
            service.insertNewPasswordEntry('uNHash', 'pWHash', 'some.url', date, 1234, 5678, 3, 'UULLSN', 'entropy', 'crackTime', []);
            dbRequest[0].onsuccess();
            rootScope.$digest();
            expect(newEntry.metrics).toEqual({
                passwordHash: 'pWHash',
                length: 5678,
                numOfCharClasses: 3,
                structure: 'UULLSN',
                entropy: 'entropy',
                crackTime: 'crackTime'
            });
        });

        it('should successfully put variations into password variations store', function () {
            service.insertNewPasswordEntry('uNHash', 'pWHash', 'some.url', date, 1234, 5678, 3, 'UULLSN', 'entropy', 'crackTime', [
                {
                    variationHash: 'abc',
                    reason: ['reason1', 'reason2']
                }, {
                    variationHash: '123',
                    reason: ['reason2']
                }
            ]);
            dbRequest[0].onsuccess();
            rootScope.$digest();
            expect(newEntry.passwordVariations).toEqual([
                {
                    passwordHash: 'pWHash',
                    variationHash: 'abc',
                    reason: ['reason1', 'reason2']
                }, {
                    passwordHash: 'pWHash',
                    variationHash: '123',
                    reason: ['reason2']
                }
            ]);
        });

        it('should not put anything into any store if any information is missing', function () {
            service.insertNewPasswordEntry('', '', '', '', '', '', '', '', '', '', []);
            expect(newEntry).toBeUndefined();
        });
    });

    describe('insertNewUsageEntry', function () {
        var date = new Date().getTime();
        it('should successfully put new entry into credentialUses store with correct previous retry', function () {
            service.insertNewUsageEntry('uNHash', 'pWHash', 'some.url', date, 1234);
            for (var i = 0; i < dbRequest.length; i++) {
                dbRequest[i].onsuccess();
            }
            rootScope.$apply();
            request.onsuccess();
            rootScope.$apply();
            expect(newEntry.credentialUses).toEqual({
                usernameHash: 'uNHash',
                passwordHash: 'pWHash',
                url: 'some.url',
                time: date,
                timeSpent: 1234,
                prevRetry: 'previous retry'
            });
        });

        it('should not put in anything if some value is missing', function () {
            service.insertNewUsageEntry('', 'pWHash', 'some.url', date, 1234);
            expect(dbRequest.length).toBe(0);
            expect(newEntry).toBeUndefined();
        });
    });

    describe('insertNewOAuthEntry', function () {
        var date = new Date().getTime();
        it('should successfully insert new entry into oauth store', function () {
            service.insertNewOAuthEntry('some.provider.url', 'some.url', date);
            dbRequest[0].onsuccess();
            rootScope.$apply();
            expect(newEntry.oauth).toEqual({
                oauthProviderUrl: 'some.provider.url',
                url: 'some.url',
                time: date
            });
        });

        it('should not put in anything if any value is missing', function () {
            service.insertNewOAuthEntry('some.provider.url', 'some.url');
            expect(dbRequest.length).toBe(0);
            expect(newEntry).toBeUndefined();
        })
    });

    describe('deleteCheckedEntries', function () {
        it('should successfully delete checked entries', function () {
            var callback = jasmine.createSpy('callback');
            service.deleteCheckedEntries([
                {
                    passwordHash: 'pHash1',
                    usernameHash: 'uHash1',
                    url: 'some.url.1',
                    time: date,
                    checked: true
                }, {
                    passwordHash: 'pHash2',
                    usernameHash: 'uHash2',
                    url: 'some.url.2',
                    time: date,
                    checked: false
                }
            ], callback);
            dbRequest[0].onsuccess();
            rootScope.$apply();
            request.onsuccess();
            rootScope.$apply();
            expect(newEntry.deletedEntries).toEqual({
                timeDeleted: jasmine.any(Number),
                time: date,
                hashedUrl: 'hashed'
            });
            request.onsuccess();
            expect(deleteEntry).toEqual(['uHash1', 'pHash1', 'some.url.1', date]);
            trans.oncomplete();
            expect(callback).toHaveBeenCalled();
        });

        it('should not put in anything if list is empty or missing', function () {
            var callback = jasmine.createSpy('callback');
            service.deleteCheckedEntries([], callback);
            expect(dbRequest.length).toBe(0);
            expect(deleteEntry).toBeUndefined();
            expect(newEntry).toBeUndefined();
            expect(callback).toHaveBeenCalled();
            service.deleteCheckedEntries(undefined, callback);
            expect(dbRequest.length).toBe(0);
            expect(deleteEntry).toBeUndefined();
            expect(newEntry).toBeUndefined();
            expect(callback).toHaveBeenCalled();
        })
    });

    describe('deleteEntriesFromUrl', function () {
        it("should correctly delete entries given a URL", function () {
            service.deleteEntriesFromUrl('some.url');
            dbRequest[0].onsuccess();
            rootScope.$apply();
            request.onsuccess();
            request.onsuccess();
            rootScope.$apply();
            request.onsuccess();
            expect(deleteEntry).toEqual(['uh', 'ph', 'some.url', date]);
        });

        it("should not delete anything if URL is empty or missing", function () {
            service.deleteEntriesFromUrl('');
            expect(dbRequest.length).toBe(0);
            expect(deleteEntry).toBeUndefined();
            expect(newEntry).toBeUndefined();
            service.deleteEntriesFromUrl();
            expect(dbRequest.length).toBe(0);
            expect(deleteEntry).toBeUndefined();
            expect(newEntry).toBeUndefined();
        });
    });

    describe("getAllTabled", function () {
        it("should successfully get all tables in an object", function (done) {
            service.getAllTables().then(function (result) {
                expect(result).toEqual({metrics: [3, 2, 1]});
                done();
            });
            dbRequest[0].onsuccess();
            rootScope.$apply();
            request.onsuccess();
            rootScope.$apply();
        });
    });
});