/**
 * Created by Nikolai on 06/04/15.
 */
describe('ManglingService', function () {

    var service;
    beforeEach(function () {
        module('PasswordMonitor');
        inject(function (ManglingService) {
            service = ManglingService
        });
    });

    it('should be defined', function () {
        expect(service).toBeDefined();
    });

    it('should expose method', function () {
        expect(service.mangle).toBeDefined();
        expect(service.mangle).toEqual(jasmine.any(Function));
    });

    describe('mangle', function () {
        it('should not return anything if password is empty', function () {
            expect(service.mangle('')).toEqual(undefined);
        });

        it('should not return anything if password is missing', function () {
            expect(service.mangle()).toEqual(undefined);
        });

        it('should successfully apply increment rule', function () {
            var variations = service.mangle('1a2');
            var i, found;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === '2a2') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === '1a3') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === '2a3') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
        });

        it('should successfully apply common substitution rule', function () {
            var variations = service.mangle('%$@');
            var i, found;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === '5$@') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === '%s@') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === '%S@') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === '%5@') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === '%$A') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === '%$a') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === '5S@') {
                    found = true;
                }
            }
            expect(found).toBeFalsy();
        });

        it('should successfully apply append symbols rule', function () {
            var variations = service.mangle('abc123');
            var i, found;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc123!') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc123+') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc123=') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc123?') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc123/') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc123\\') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc123#') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc123%') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc123&') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc123*') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc123;') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc123:') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
        });

        it('should successfully apply append current year rule', function () {
            var variations = service.mangle('abc123');
            var i, found;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc12315') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'abc1232015') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === '@bc12315') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
            found = false;

            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === '@bc1232015') {
                    found = true;
                }
            }
            expect(found).toBeTruthy();
        });

        it('should not return the password itself', function () {
            var variations = service.mangle('password123');
            var i, found;
            for (i = 0; i < variations.length; i++) {
                if (variations[i].variation === 'password123') {
                    found = true;
                }
            }
            expect(found).toBeFalsy();
        });
    })
});