/**
 * Created by Nikolai on 06/04/15.
 */
describe('MonitorService', function () {

    var service, idbService, hashingService, timerService, q, rootScope, time;
    beforeAll(function () {
        time = new Date().getTime();
    });
    beforeEach(function () {
        module('PasswordMonitor', function ($provide) {
            $provide.decorator('$window', function ($delegate) {
                $delegate.chrome = {
                    storage: {
                        local: {
                            get: function () {
                            }
                        }
                    },
                    webRequest: {
                        onBeforeRequest: {
                            addListener: function () {
                            }
                        }
                    },
                    tabs: {
                        get: function () {
                        }
                    }
                };
                return $delegate;
            })
        });
        inject(function (MonitorService, IdbService, HashingService, TimerService, $rootScope, $q) {
            service = MonitorService;
            idbService = IdbService;
            hashingService = HashingService;
            timerService = TimerService;
            rootScope = $rootScope;
            q = $q;
        });
        spyOn(hashingService, 'hashThis').and.callFake(function (cleartext) {
            var deferred = q.defer();
            if (cleartext === 'username') {
                deferred.resolve('uHash');
            }
            else if (cleartext === 'password') {
                deferred.resolve('pHash');
            }
            else {
                deferred.resolve('variationHash');
            }
            return deferred.promise;
        });
        spyOn(window.chrome.storage.local, 'get').and.callFake(function (key, callback) {
            expect(key).toEqual('exclusionList');
            callback({exclusionList: []});
        });
    });

    it("should be defined and have methods exposed", function () {
        expect(service).toBeDefined();
        expect(service.newPassword).toBeDefined();
        expect(service.newPassword).toEqual(jasmine.any(Function));
    });

    describe("newPassword", function () {
        it("should call IdbService.insertNewUsageEntry", function () {
            spyOn(idbService, 'insertNewUsageEntry');
            service.newPassword('some.url', 'username', 'password', time, 1234);
            rootScope.$apply();
            expect(idbService.insertNewUsageEntry).toHaveBeenCalledWith('uHash', 'pHash', 'some.url', time, 1234);
        });

        it("should call IdbService.insertNewPasswordEntry if password does not exist before", function () {
            spyOn(idbService, 'hasPasswordAlready').and.callFake(function () {
                var deferred = q.defer();
                deferred.resolve(false);
                return deferred.promise
            });
            spyOn(idbService, 'insertNewPasswordEntry');
            service.newPassword('some.url', 'username', 'password', time, 1234);
            rootScope.$apply();
            expect(idbService.insertNewPasswordEntry).toHaveBeenCalledWith('uHash', 'pHash', 'some.url', time, 1234, 8,
                1, 'LLLLLLLL', jasmine.any(Number), jasmine.any(String), jasmine.any(Array));
        });

        it("should not call IdbService.insertNewPasswordEntry if password exists in DB", function () {
            spyOn(idbService, 'hasPasswordAlready').and.callFake(function () {
                var deferred = q.defer();
                deferred.resolve(true);
                return deferred.promise;
            });
            spyOn(idbService, 'insertNewPasswordEntry');
            service.newPassword('some.url', 'username', 'password', time, 1234);
            rootScope.$apply();
            expect(idbService.insertNewPasswordEntry).not.toHaveBeenCalled();
        });

        it("should call insertNewUsageEntry and insertNewPasswordEntry with 'unknown' if it is", function () {
            spyOn(idbService, 'hasPasswordAlready').and.callFake(function () {
                var deferred = q.defer();
                deferred.resolve(false);
                return deferred.promise;
            });
            spyOn(idbService, 'insertNewUsageEntry');
            spyOn(idbService, 'insertNewPasswordEntry');
            service.newPassword('some.url', 'unknown', 'password', time, 1234);
            rootScope.$apply();
            expect(idbService.insertNewUsageEntry).toHaveBeenCalledWith('unknown', 'pHash', 'some.url', time, 1234);
            expect(idbService.insertNewPasswordEntry).toHaveBeenCalledWith('unknown', 'pHash', 'some.url', time, 1234,
                8, 1, 'LLLLLLLL', jasmine.any(Number), jasmine.any(String), jasmine.any(Array));
        });

        it("should not call neither insertNewUsageEntry nor insertNewPasswordEntry if any field is missing", function () {
            spyOn(idbService, 'hasPasswordAlready').and.callFake(function () {
                var deferred = q.defer();
                deferred.resolve(false);
                return deferred.promise;
            });
            spyOn(idbService, 'insertNewUsageEntry');
            spyOn(idbService, 'insertNewPasswordEntry');
            service.newPassword('some.url', '', 'password', time, 1234);
            rootScope.$apply();
            expect(idbService.insertNewUsageEntry).not.toHaveBeenCalled();
            expect(idbService.insertNewPasswordEntry).not.toHaveBeenCalled();
            service.newPassword('some.url', 'username', '', time, 1234);
            rootScope.$apply();
            expect(idbService.insertNewUsageEntry).not.toHaveBeenCalled();
            expect(idbService.insertNewPasswordEntry).not.toHaveBeenCalled();
            service.newPassword('some.url', 'username', 'password', time, 0);
            rootScope.$apply();
            expect(idbService.insertNewUsageEntry).not.toHaveBeenCalled();
            expect(idbService.insertNewPasswordEntry).not.toHaveBeenCalled();
        });

        it("should not call neither insertNewUsageEntry nor insertNewPasswordEntry if suspended", function () {
            spyOn(timerService, 'isSuspended').and.returnValue(true);
            spyOn(idbService, 'insertNewUsageEntry');
            spyOn(idbService, 'insertNewPasswordEntry');
            service.newPassword('some.url', 'username', 'password', time, 1234);
            rootScope.$apply();
            expect(idbService.insertNewUsageEntry).not.toHaveBeenCalled();
            expect(idbService.insertNewPasswordEntry).not.toHaveBeenCalled();
        });
    });
});