describe('DataUploadModalCtrl', function () {

    var ctrl, scope, window, callback, returnObject = {
        onMessage: {
            addListener: function (listener) {
                "use strict";
                callback = listener;
            }
        }
    };
    beforeEach(function () {
        module('PasswordMonitor');
        inject(function ($controller, $rootScope, $window) {
            scope = $rootScope.$new();
            window = $window;
            ctrl = $controller('DataUploadModalCtrl', {
                $scope: scope
            });
        });
    });

    it('should have scope objects instantiated', function () {
        expect(scope.stage).toBeDefined();
        expect(scope.startUpload).toBeDefined();
        expect(scope.startUpload).toEqual(jasmine.any(Function));
    });

    it('should set inProgress to true when startUpload is called', function () {
        expect(scope.inProgress).toBeFalsy();
        window.chrome.runtime.connect = function () {
            return returnObject;
        };
        scope.startUpload();
        expect(scope.inProgress).toBeTruthy();
    });

    describe('return message', function () {

        beforeEach(function () {
            window.chrome.runtime.connect = function () {
                return returnObject;
            };
            scope.startUpload();
        });

        it('should set right flags when in progress', function () {
            callback({
                inProgress: true,
                stage: 1,
                text: "Some text",
                percentage: 50
            });
            expect(scope.inProgress).toBeTruthy();
            expect(scope.stage).toEqual(1);
            expect(scope.statusText).toEqual("Some text");
            expect(scope.percentage).toEqual(50);
        });

        it('should be able to handle that message parts are missing', function () {
            callback({
                inProgress: true
            });
            expect(scope.inProgress).toBeTruthy();
            expect(scope.stage).toEqual(0);
            expect(scope.statusText).toBeUndefined();
            expect(scope.percentage).toBeUndefined();
        });

        it('should set correctly when successful', function () {
            callback({
                success: true
            });
            expect(scope.inProgress).toBeFalsy();
            expect(scope.success).toBeTruthy();
        });

        it('should handle errors correctly', function () {
            callback({
                errorCode: 44,
                error: "some error"
            });
            expect(scope.inProgress).toBeFalsy();
            expect(scope.error).toBeTruthy();
            expect(scope.errorCode).toEqual(44);
            expect(scope.errorText).toEqual("some error");
        });

        it('should not do anything when nothing in the message', function () {
            callback();
            expect(scope.inProgress).toBeTruthy();
            expect(scope.stage).toEqual(0);
        })
    })
});