describe('SettingsCtrl', function () {

    var ctrl, scope, listener, modalConfig, modalCallback, storageKey, storageCallback, setObject, searchConfig,
        searchCallback, time, stopTimerCalled;
    var messages = [], msgCallbacks = [];
    var modal = {
        open: function (config) {
            modalConfig = config;
            return {
                result: {
                    then: function (callback) {
                        modalCallback = callback;
                    }
                }
            }
        }
    };
    var timerService = {
        startTimer: function (setTime) {
            time = setTime;
        },
        stopTimer: function () {
            stopTimerCalled = true;
        },
        isSuspended: Function,
        getMinutes: Function,
        getSeconds: Function,
        getPercent: Function
    };

    beforeEach(function () {
        time = undefined;
        messages = [];
        msgCallbacks = [];
        module('PasswordMonitor');
        inject(function ($controller, $rootScope, $window) {
            scope = $rootScope.$new();
            $window.chrome = {
                runtime: {
                    sendMessage: function (message, callback) {
                        messages.push(message);
                        msgCallbacks.push(callback);
                    },
                    onMessage: {
                        addListener: function (ltnr) {
                            listener = ltnr;
                        }
                    },
                    getURL: function () {
                        return 'something.html'
                    }
                },
                storage: {
                    local: {
                        get: function (key, callback) {
                            storageKey = key;
                            storageCallback = callback;
                        },
                        set: function (object) {
                            setObject = object;
                        }
                    }
                },
                history: {
                    search: function (config, callback) {
                        searchConfig = config;
                        searchCallback = callback;
                    }
                }
            };

            ctrl = $controller('SettingsCtrl', {
                $scope: scope,
                $modal: modal,
                TimerService: timerService
            });
        });
    });

    it('should have scope objects instantiated', function () {
        expect(scope.button).toBeDefined();
        expect(scope.button.uploading).toBeFalsy();
        expect(scope.button.text).toBe("Upload Data");

        expect(scope.uploadData).toBeDefined();
        expect(scope.uploadData).toEqual(jasmine.any(Function));

        expect(scope.time).toBeDefined();
        expect(scope.time.options).toBeDefined();
        expect(scope.time.options.size).toBe(150);
        expect(scope.time.options.size).toBe(150);
        expect(scope.time.isSuspended).toBeDefined();
        expect(scope.time.isSuspended).toEqual(jasmine.any(Function));
        expect(scope.time.minutes).toBeDefined();
        expect(scope.time.minutes).toEqual(jasmine.any(Function));
        expect(scope.time.seconds).toBeDefined();
        expect(scope.time.seconds).toEqual(jasmine.any(Function));
        expect(scope.time.percent).toBeDefined();
        expect(scope.time.percent).toEqual(jasmine.any(Function));
        expect(scope.startTimer).toBeDefined();
        expect(scope.startTimer).toEqual(jasmine.any(Function));
        expect(scope.alerts).toBeDefined();
        expect(scope.alerts).toEqual(jasmine.any(Array));
        expect(scope.alerts.length).toEqual(0);
        expect(scope.startCustomTimer).toBeDefined();
        expect(scope.startCustomTimer).toEqual(jasmine.any(Function));
        expect(scope.closeAlert).toBeDefined();
        expect(scope.closeAlert).toEqual(jasmine.any(Function));

        expect(scope.exclusionList).toBeDefined();
        expect(scope.exclusionList).toEqual(jasmine.any(Array));
        expect(scope.exclusionList.length).toEqual(0);
        expect(scope.history).toBeDefined();
        expect(scope.history).toEqual(jasmine.any(Array));
        expect(scope.history.length).toEqual(0);
        expect(scope.refreshExclusionList).toBeDefined();
        expect(scope.refreshExclusionList).toEqual(jasmine.any(Function));
        expect(scope.addToExclusionList).toBeDefined();
        expect(scope.addToExclusionList).toEqual(jasmine.any(Function));
        expect(scope.removeFromExclusionList).toBeDefined();
        expect(scope.removeFromExclusionList).toEqual(jasmine.any(Function));

        expect(scope.passwordMetrics).toBeDefined();
        expect(scope.passwordMetrics).toEqual(jasmine.any(Array));
        expect(scope.passwordMetrics.length).toEqual(0);
        expect(scope.credentialUses).toBeDefined();
        expect(scope.credentialUses).toEqual(jasmine.any(Array));
        expect(scope.credentialUses.length).toEqual(0);
        expect(scope.toggleMasterCheck).toBeDefined();
        expect(scope.toggleMasterCheck).toEqual(jasmine.any(Function));
        expect(scope.anyChecked).toBeDefined();
        expect(scope.anyChecked).toEqual(jasmine.any(Function));
        expect(scope.deleteEntries).toBeDefined();
        expect(scope.deleteEntries).toEqual(jasmine.any(Function));
        expect(scope.refresh).toBeDefined();
        expect(scope.refresh).toEqual(jasmine.any(Function));

        expect(scope.date).toBeDefined();
        expect(scope.date).toEqual(jasmine.any(Date));

        expect(scope.clear).toBeDefined();
        expect(scope.clear).toEqual(jasmine.any(Function));
        expect(scope.hash).toBeDefined();
        expect(scope.hash).toEqual(jasmine.any(Function));

        expect(scope.openChromeExtensions).toBeDefined();
        expect(scope.openChromeExtensions).toEqual(jasmine.any(Function));
    });

    it('should call the correct Google API functions to begin with', function () {
        expect(messages[0]).toEqual({'type': 'getSalt'});
        expect(messages[1]).toEqual({'type': 'getTime'});
        expect(searchConfig).toEqual({text: "", maxResults: 500});
        expect(messages[2]).toEqual({'type': 'getPasswordMetrics'});
        expect(messages[3]).toEqual({'type': 'getCredentialUses'});
        expect(listener).toBeDefined();
        expect(listener).toEqual(jasmine.any(Function));
    });

    it('should open modal if salt is returned', function () {
        expect(messages[0]).toEqual({'type': 'getSalt'});
        msgCallbacks[0]();
        expect(modalConfig).toEqual({
            templateUrl: 'something.html',
            controller: 'UserIdModalCtrl',
            backdrop: 'static'
        });
    });

    it('should provide TimerService with right time if time returns', function () {
        expect(messages[1]).toEqual({'type': 'getTime'});
        msgCallbacks[1]('someTime');
        expect(time).toEqual('someTime');
    });

    it('should not provide TimerService with right times if time doesn\'t return', function () {
        expect(messages[1]).toEqual({'type': 'getTime'});
        msgCallbacks[1]();
        expect(time).toBeUndefined();
    });

    it('should set exclusionList if storage returns successfully', function () {
        expect(storageKey).toBe('exclusionList');
        storageCallback({exclusionList: ['some', 'list']});
        expect(scope.exclusionList).toEqual(['some', 'list']);
    });

    it('should not set exclusionList if storage returns error', function () {
        expect(storageKey).toBe('exclusionList');
        storageCallback({});
        expect(scope.exclusionList.length).toBe(0);
    });

    describe('$scope.history', function () {
        it('should contain all canonical URLs if they return', function () {
            expect(searchConfig).toEqual({text: "", maxResults: 500});
            searchCallback([
                {url: 'http://blog.fourtonfish.com/post/74055065673/dynamically-add-directives-in-angularjs-no-jquery'},
                {url: 'https://material.angularjs.org/#/demo/material.components.chips'},
                {url: 'http://fdietz.github.io/recipes-with-angular-js/directives/changing-the-dom-in-response-to-user-actions.html'}
            ]);
            expect(scope.history).toEqual([
                'blog.fourtonfish.com',
                'material.angularjs.org',
                'fdietz.github.io'
            ]);
        });

        it('should only contain unique URLs', function () {
            expect(searchConfig).toEqual({text: "", maxResults: 500});
            searchCallback([
                {url: 'https://material.angularjs.org/#/demo/material.components.chips'},
                {url: 'https://material.angularjs.org/#/demo/material.components.chips'}
            ]);
            expect(scope.history).toEqual([
                'material.angularjs.org'
            ]);
        })
    });

    it('should set passwordMetrics if it returns correctly', function () {
        expect(messages[2]).toEqual({'type': 'getPasswordMetrics'});
        msgCallbacks[2]({passwordMetrics: ['some', 'password', 'metrics']});
        expect(scope.passwordMetrics).toEqual(['some', 'password', 'metrics']);
    });

    it('should set credentialUses if it returns correctly', function () {
        expect(messages[3]).toEqual({'type': 'getCredentialUses'});
        msgCallbacks[3]({credentialUses: ['some', 'credential', 'uses']});
        expect(scope.credentialUses).toEqual(['some', 'credential', 'uses']);
    });

    describe('uploadData', function () {
        it('should set correct button values', function () {
            expect(scope.button.uploading).toBeFalsy();
            expect(scope.button.text).toEqual('Upload Data');
            scope.uploadData();
            expect(scope.button.uploading).toBeTruthy();
            expect(scope.button.text).toEqual('Uploading...');

        });

        it('should open modal', function () {
            scope.uploadData();
            expect(modalConfig).toEqual({
                templateUrl: 'something.html',
                controller: 'DataUploadModalCtrl',
                backdrop: 'static'
            });
        });

        it('should reset button when modal returns', function () {
            scope.uploadData();
            expect(scope.button.uploading).toBeTruthy();
            expect(scope.button.text).toEqual('Uploading...');
            expect(modalCallback).toBeDefined();
            modalCallback();
            expect(scope.button.uploading).toBeFalsy();
            expect(scope.button.text).toEqual('Upload Data');

        });
    });

    describe('startTimer', function () {
        it('should send startTimer message', function () {
            scope.startTimer(5);
            expect(messages[4].type).toBe('startTimer');
            expect(messages[4].time).toEqual({
                startTime: 5,
                totalSeconds: 5
            });
        });

        it('should send startTimer with 0 if no argument is supplied', function () {
            scope.startTimer();
            expect(messages[4].time).toEqual({
                startTime: 0,
                totalSeconds: 0
            });
        })
    });

    describe('startCustomTimer', function () {
        var answer = ((5 * 60) + 40) * 60;
        it('should call startTimer if valid', function () {
            scope.customTime = {
                $valid: true
            };
            scope.startCustomTimer({
                hr: 5,
                min: 40
            });
            expect(messages[4].time).toEqual({
                startTime: answer,
                totalSeconds: answer
            });
            expect(scope.alerts.length).toBe(0);
        });

        it('should push an alert if invalid', function () {
            scope.customTime = {
                $valid: false
            };
            scope.startCustomTimer({
                hr: 5,
                min: 40
            });
            expect(messages[4]).toBeUndefined();
            expect(scope.alerts.length).toBe(1);
            expect(scope.alerts).toContain('Cannot suspend for more than 11:59');
        });

        it('should remove the alert when it is dismissed', function () {
            scope.customTime = {
                $valid: false
            };
            scope.startCustomTimer({
                hr: 5,
                min: 40
            });
            expect(scope.alerts.length).toBe(1);
            scope.closeAlert(0);
            expect(scope.alerts.length).toBe(0);
        })
    });

    describe('exclusion list', function () {
        it('should successfully add name to list', function () {
            expect(scope.exclusionList.length).toBe(0);
            scope.addToExclusionList('somewebsite.com');
            expect(scope.exclusionList.length).toBe(1);
            expect(scope.exclusionList).toContain('somewebsite.com');
            expect(setObject).toEqual({'exclusionList': ['somewebsite.com']});
            expect(scope.newExclusion).toBe('');
        });

        it('should successfully handle existing entry when adding', function () {
            scope.exclusionList.push('existing.entry');
            expect(scope.exclusionList.length).toBe(1);
            scope.addToExclusionList('somewebsite.com');
            expect(scope.exclusionList.length).toBe(2);
            expect(scope.exclusionList).toContain('existing.entry');
            expect(scope.exclusionList).toContain('somewebsite.com');
            expect(setObject).toEqual({'exclusionList': ['existing.entry', 'somewebsite.com']});
            expect(scope.newExclusion).toBe('');
        });

        it('should not add anything if argument is empty', function () {
            expect(scope.exclusionList.length).toBe(0);
            scope.addToExclusionList('');
            expect(scope.exclusionList.length).toBe(0);
        });

        it('should not add anything if argument is missing', function () {
            expect(scope.exclusionList.length).toBe(0);
            scope.addToExclusionList();
            expect(scope.exclusionList.length).toBe(0);
        });

        it('should successfully remove entry', function () {
            scope.exclusionList.push('existing.entry');
            expect(scope.exclusionList.length).toBe(1);
            scope.removeFromExclusionList(0);
            expect(scope.exclusionList.length).toBe(0);
            expect(setObject).toEqual({'exclusionList': []});
        });

        it('should not remove anything when index is invalid', function () {
            scope.exclusionList.push('existing.entry');
            expect(scope.exclusionList.length).toBe(1);
            scope.removeFromExclusionList(-3);
            expect(scope.exclusionList.length).toBe(1);
        });

        it('should not remove anything when index is missing', function () {
            scope.exclusionList.push('existing.entry');
            expect(scope.exclusionList.length).toBe(1);
            scope.removeFromExclusionList();
            expect(scope.exclusionList.length).toBe(1);
        })
    });

    describe('toggleMasterCheck', function () {
        it('should check all uses for a passwordHash when set to true', function () {
            scope.credentialUses.push({passwordHash: 'hash1'}, {passwordHash: 'hash2'}, {passwordHash: 'hash1'});
            expect(scope.credentialUses.length).toBe(3);
            expect(scope.credentialUses[0].checked).toBeFalsy();
            expect(scope.credentialUses[1].checked).toBeFalsy();
            expect(scope.credentialUses[2].checked).toBeFalsy();
            scope.toggleMasterCheck('hash1', true);
            expect(scope.credentialUses[0].checked).toBeTruthy();
            expect(scope.credentialUses[1].checked).toBeFalsy();
            expect(scope.credentialUses[2].checked).toBeTruthy();
            scope.toggleMasterCheck('hash1', false);
            expect(scope.credentialUses[0].checked).toBeFalsy();
            expect(scope.credentialUses[1].checked).toBeFalsy();
            expect(scope.credentialUses[2].checked).toBeFalsy();
        });

        it('should provide no change when already checked', function () {
            scope.credentialUses.push(
                {passwordHash: 'hash1', checked: true},
                {passwordHash: 'hash2'},
                {passwordHash: 'hash1', checked: true}
            );
            expect(scope.credentialUses[0].checked).toBeTruthy();
            expect(scope.credentialUses[1].checked).toBeFalsy();
            expect(scope.credentialUses[2].checked).toBeTruthy();
            expect(scope.credentialUses.length).toBe(3);
            scope.toggleMasterCheck('hash1', true);
            expect(scope.credentialUses[0].checked).toBeTruthy();
            expect(scope.credentialUses[1].checked).toBeFalsy();
            expect(scope.credentialUses[2].checked).toBeTruthy();
        });

        it('should NO-OP when passwordHash arg is empty', function () {
            scope.credentialUses.push({passwordHash: 'hash1'}, {passwordHash: 'hash2'}, {passwordHash: 'hash1'});
            expect(scope.credentialUses.length).toBe(3);
            expect(scope.credentialUses[0].checked).toBeFalsy();
            expect(scope.credentialUses[1].checked).toBeFalsy();
            expect(scope.credentialUses[2].checked).toBeFalsy();
            scope.toggleMasterCheck('', true);
            expect(scope.credentialUses[0].checked).toBeFalsy();
            expect(scope.credentialUses[1].checked).toBeFalsy();
            expect(scope.credentialUses[2].checked).toBeFalsy();
        });

        it('should NO-OP when passwordHash arg is undefined', function () {
            scope.credentialUses.push({passwordHash: 'hash1'}, {passwordHash: 'hash2'}, {passwordHash: 'hash1'});
            expect(scope.credentialUses.length).toBe(3);
            expect(scope.credentialUses[0].checked).toBeFalsy();
            expect(scope.credentialUses[1].checked).toBeFalsy();
            expect(scope.credentialUses[2].checked).toBeFalsy();
            scope.toggleMasterCheck(undefined, true);
            expect(scope.credentialUses[0].checked).toBeFalsy();
            expect(scope.credentialUses[1].checked).toBeFalsy();
            expect(scope.credentialUses[2].checked).toBeFalsy();
        })
    });

    describe('anyChecked', function () {
        it('should return true when all are checked', function () {
            scope.credentialUses.push(
                {passwordHash: 'hash1', checked: true},
                {passwordHash: 'hash2', checked: true},
                {passwordHash: 'hash1', checked: true}
            );
            expect(scope.credentialUses[0].checked).toBeTruthy();
            expect(scope.credentialUses[1].checked).toBeTruthy();
            expect(scope.credentialUses[2].checked).toBeTruthy();
            expect(scope.anyChecked()).toBeTruthy();
        });

        it('should return true when some are checked', function () {
            scope.credentialUses.push(
                {passwordHash: 'hash1', checked: true},
                {passwordHash: 'hash2'},
                {passwordHash: 'hash1', checked: true}
            );
            expect(scope.credentialUses[0].checked).toBeTruthy();
            expect(scope.credentialUses[1].checked).toBeFalsy();
            expect(scope.credentialUses[2].checked).toBeTruthy();
            expect(scope.anyChecked()).toBeTruthy();
        });

        it('should return false when none are checked', function () {
            scope.credentialUses.push({passwordHash: 'hash1'}, {passwordHash: 'hash2'}, {passwordHash: 'hash1'});
            expect(scope.credentialUses[0].checked).toBeFalsy();
            expect(scope.credentialUses[1].checked).toBeFalsy();
            expect(scope.credentialUses[2].checked).toBeFalsy();
            expect(scope.anyChecked()).toBeFalsy();
        })
    });

    describe('refresh', function () {
        it('should send getPasswordMetrics and getCredentialUses messages', function () {
            scope.refresh();
            expect(messages[4].type).toBe('getPasswordMetrics');
            expect(messages[5].type).toBe('getCredentialUses');
        });

        it('should replace passwordMetrics and credentialUses with new valued when callbacks return', function () {
            scope.passwordMetrics.push({some: 'value1'}, {some: 'value2'}, {some: 'value3'});
            expect(scope.passwordMetrics[0]).toEqual({some: 'value1'});
            expect(scope.passwordMetrics[1]).toEqual({some: 'value2'});
            expect(scope.passwordMetrics[2]).toEqual({some: 'value3'});
            scope.credentialUses.push({passwordHash: 'hash1'}, {passwordHash: 'hash2'}, {passwordHash: 'hash1'});
            expect(scope.credentialUses[0]).toEqual({passwordHash: 'hash1'});
            expect(scope.credentialUses[1]).toEqual({passwordHash: 'hash2'});
            expect(scope.credentialUses[2]).toEqual({passwordHash: 'hash1'});
            scope.refresh();
            msgCallbacks[4]({passwordMetrics: [{some: 'other1'}, {some: 'other2'}, {some: 'other3'}]});
            msgCallbacks[5]({
                credentialUses: [{passwordHash: 'hash3'}, {passwordHash: 'hash4'}, {passwordHash: 'hash5'}]
            });
            expect(scope.passwordMetrics[0]).toEqual({some: 'other1'});
            expect(scope.passwordMetrics[1]).toEqual({some: 'other2'});
            expect(scope.passwordMetrics[2]).toEqual({some: 'other3'});
            expect(scope.credentialUses[0]).toEqual({passwordHash: 'hash3'});
            expect(scope.credentialUses[1]).toEqual({passwordHash: 'hash4'});
            expect(scope.credentialUses[2]).toEqual({passwordHash: 'hash5'});
        });

        it('should remember which passwordMetrics that were open and maintain them after refresh', function () {
            scope.passwordMetrics.push(
                {passwordHash: 'hash1', open: true},
                {passwordHash: 'hash2'},
                {passwordHash: 'hash3', open: true});
            expect(scope.passwordMetrics[0].passwordHash).toEqual('hash1');
            expect(scope.passwordMetrics[1].passwordHash).toEqual('hash2');
            expect(scope.passwordMetrics[2].passwordHash).toEqual('hash3');
            expect(scope.passwordMetrics[0].open).toBeTruthy();
            expect(scope.passwordMetrics[1].open).toBeFalsy();
            expect(scope.passwordMetrics[2].open).toBeTruthy();
            scope.refresh();
            msgCallbacks[4]({
                passwordMetrics: [
                    {passwordHash: 'hash1'},
                    {passwordHash: 'hash4'},
                    {passwordHash: 'hash5'},
                    {passwordHash: 'hash3'}
                ]
            });
            expect(scope.passwordMetrics[0].passwordHash).toEqual('hash1');
            expect(scope.passwordMetrics[1].passwordHash).toEqual('hash4');
            expect(scope.passwordMetrics[2].passwordHash).toEqual('hash5');
            expect(scope.passwordMetrics[3].passwordHash).toEqual('hash3');
            expect(scope.passwordMetrics[0].open).toBeTruthy();
            expect(scope.passwordMetrics[1].open).toBeFalsy();
            expect(scope.passwordMetrics[2].open).toBeFalsy();
            expect(scope.passwordMetrics[3].open).toBeTruthy();
        })
    });

    describe('deleteEntries', function () {
        it('should successfully send delete message', function () {
            scope.credentialUses.push({passwordHash: 'hash1'}, {passwordHash: 'hash2'}, {passwordHash: 'hash1'});
            scope.deleteEntries();
            expect(messages[4].type).toBe('deleteCheckedEntries');
            expect(messages[4].credentialUses).toEqual([
                {passwordHash: 'hash1'},
                {passwordHash: 'hash2'},
                {passwordHash: 'hash1'}
            ]);
        });

        it('should call refresh when callback returns', function () {
            scope.deleteEntries();
            msgCallbacks[4]();
            expect(messages[5].type).toBe('getPasswordMetrics');
            expect(messages[6].type).toBe('getCredentialUses');
        })
    });

    describe('clear', function () {
        it('should unmatch all passwordMetrics if some are matched', function () {
            scope.passwordMetrics.push(
                {passwordHash: 'hash1', matched: true},
                {passwordHash: 'hash2'},
                {passwordHash: 'hash3', matched: true});
            expect(scope.passwordMetrics[0].matched).toBeTruthy();
            expect(scope.passwordMetrics[1].matched).toBeFalsy();
            expect(scope.passwordMetrics[2].matched).toBeTruthy();
            scope.clear();
            expect(scope.passwordMetrics[0].matched).toBeFalsy();
            expect(scope.passwordMetrics[1].matched).toBeFalsy();
            expect(scope.passwordMetrics[2].matched).toBeFalsy();
        });

        it('should unmatch all passwordMetrics if one are matched', function () {
            scope.passwordMetrics.push(
                {passwordHash: 'hash1'},
                {passwordHash: 'hash2'},
                {passwordHash: 'hash3', matched: true});
            expect(scope.passwordMetrics[0].matched).toBeFalsy();
            expect(scope.passwordMetrics[1].matched).toBeFalsy();
            expect(scope.passwordMetrics[2].matched).toBeTruthy();
            scope.clear();
            expect(scope.passwordMetrics[0].matched).toBeFalsy();
            expect(scope.passwordMetrics[1].matched).toBeFalsy();
            expect(scope.passwordMetrics[2].matched).toBeFalsy();
        });

        it('should NO-OP if none are matched', function () {
            scope.passwordMetrics.push(
                {passwordHash: 'hash1', matched: false},
                {passwordHash: 'hash2', matched: false},
                {passwordHash: 'hash3', matched: false});
            expect(scope.passwordMetrics).toEqual([
                {passwordHash: 'hash1', matched: false},
                {passwordHash: 'hash2', matched: false},
                {passwordHash: 'hash3', matched: false}
            ]);
            scope.clear();
            expect(scope.passwordMetrics).toEqual([
                {passwordHash: 'hash1', matched: false},
                {passwordHash: 'hash2', matched: false},
                {passwordHash: 'hash3', matched: false}
            ]);
        });
    });

    describe('hash', function () {
        it('should set hashing to true, call $scope.clear() and send message', function () {
            scope.passwordMetrics.push({passwordHash: 'hash', matched: true});
            scope.hash('cleartext');
            expect(scope.hashing).toBeTruthy();
            expect(scope.passwordMetrics[0].matched).toBeFalsy();
            expect(messages[4]).toEqual({'type': 'hashThis', 'cleartext': 'cleartext'});
        });

        it('should NO-OP when hashing arg is empty', function () {
            scope.passwordMetrics.push({passwordHash: 'hash', matched: true});
            scope.hash('');
            expect(scope.hashing).toBeFalsy();
            expect(scope.passwordMetrics[0].matched).toBeTruthy();
            expect(messages[4]).toBeUndefined();
        });

        it('should NO-OP when hashing arg is missing', function () {
            scope.passwordMetrics.push({passwordHash: 'hash', matched: true});
            scope.hash();
            expect(scope.hashing).toBeFalsy();
            expect(scope.passwordMetrics[0].matched).toBeTruthy();
            expect(messages[4]).toBeUndefined();
        });

        it('should set the right flags when message replied', function () {
            scope.passwordMetrics.push({passwordHash: 'hashOfCleartext', matched: false});
            scope.passwordMetrics.push({passwordHash: 'notHashOfCleartext', matched: false});
            scope.hash('cleartext');
            expect(scope.hashing).toBeTruthy();
            msgCallbacks[4]('hashOfCleartext');
            expect(scope.hashResult).toBe('hashOfCleartext');
            expect(scope.hashing).toBeFalsy();
            expect(scope.passwordMetrics[0]).toEqual({passwordHash: 'hashOfCleartext', matched: true});
            expect(scope.passwordMetrics[1]).toEqual({passwordHash: 'notHashOfCleartext', matched: false});
        })
    });

    describe('openChromeExtensions', function () {
        it('should send message to open chrome://extensions', function () {
            expect(messages[4]).toBeUndefined();
            scope.openChromeExtensions();
            expect(messages[4].type).toEqual('openChromeExtensions');
        })
    });

    describe('message listener', function () {
        it('should start timer when it gets message to do so', function () {
            expect(time).toBeUndefined();
            listener({type: 'startTimer', time: 123});
            expect(time).toBe(123);
        });

        it('should not start timer when it gets message to do so and arg is missing', function () {
            expect(time).toBeUndefined();
            listener({type: 'startTimer'});
            expect(time).toBeUndefined();
        });

        it('should call stop timer when it gets message to do so', function () {
            expect(stopTimerCalled).toBeFalsy();
            listener({type: 'stopTimer'});
            expect(stopTimerCalled).toBeTruthy();
        })
    })
});