/**
 * Created by Nikolai on 02/04/15.
 */
describe('pwdFilter', function () {

    var filter;
    beforeEach(function () {
        module('PasswordMonitor');
        inject(function ($filter) {
            filter = $filter('pwdFilter');
        })
    });

    it('should have a pwdFilter', function () {
        expect(filter).toBeDefined();
        expect(filter).not.toEqual(null);
    });

    it('should correctly filter out credential uses given a password hash', function () {
        expect(filter([
            {passwordHash: 'hash1', someOther: 'value1'},
            {passwordHash: 'hash2', someOther: 'value2'},
            {passwordHash: 'hash1', someOther: 'value3'}
        ], 'hash1')).toEqual([
            {passwordHash: 'hash1', someOther: 'value1'},
            {passwordHash: 'hash1', someOther: 'value3'}
        ]);
    });

    it('should return nothing when filter value is empty', function () {
        expect(filter([
            {passwordHash: 'hash1', someOther: 'value1'},
            {passwordHash: 'hash2', someOther: 'value2'},
            {passwordHash: 'hash1', someOther: 'value3'}
        ], '')).toEqual([]);
    });

    it('should return nothing when filter value is missing', function () {
        expect(filter([
            {passwordHash: 'hash1', someOther: 'value1'},
            {passwordHash: 'hash2', someOther: 'value2'},
            {passwordHash: 'hash1', someOther: 'value3'}
        ])).toEqual([]);
    });

    it('should return nothing when list to filter is empty', function () {
        expect(filter([], 'hash1')).toEqual([]);
    });

    it('should return nothing when list to filter is missing', function () {
        expect(filter(undefined, 'hash1')).toEqual([]);
    });

    it('should return nothing when nothing matches', function () {
        expect(filter([
            {passwordHash: 'hash1', someOther: 'value1'},
            {passwordHash: 'hash2', someOther: 'value2'},
            {passwordHash: 'hash1', someOther: 'value3'}
        ], 'hash4')).toEqual([]);
    });

});