/**
 * Created by Nikolai on 02/04/15.
 */
describe('TimerService', function () {

    var service, interval;
    beforeEach(function () {
        module('PasswordMonitor');
        inject(function (TimerService, $interval) {
            service = TimerService;
            interval = $interval;
        })
    });

    it('should exist', function () {
        expect(service).toBeDefined();
    });

    it('should have a all functions', function () {
        expect(service.isSuspended).toBeDefined();
        expect(service.startTimer).toBeDefined();
        expect(service.stopTimer).toBeDefined();
        expect(service.getMinutes).toBeDefined();
        expect(service.getSeconds).toBeDefined();
        expect(service.getPercent).toBeDefined();
        expect(service.getAll).toBeDefined();
    });

    describe('isSuspended', function () {
        it('should return true if timer is ongoing', function () {
            service.startTimer({
                startTime: 10,
                totalSeconds: 10
            });
            expect(service.isSuspended()).toBeTruthy();
        });

        it('should return false is timer is stopped', function () {
            service.startTimer({
                startTime: 10,
                totalSeconds: 10
            });
            expect(service.isSuspended()).toBeTruthy();
            service.stopTimer();
            expect(service.isSuspended()).toBeFalsy();
        })
    });

    describe('startTimer', function () {
        it('should set all variables when time is set', function () {
            service.startTimer({
                startTime: 10,
                totalSeconds: 10
            });
            expect(service.getAll()).toBeTruthy();
            expect(service.getAll().startTime).toBe(10);
            expect(service.getAll().totalSeconds).toBe(10);
        });

        it('should decrement seconds correctly', function () {
            service.startTimer({
                startTime: 10,
                totalSeconds: 10
            });
            interval.flush(3000);
            expect(service.getAll().startTime).toBe(10);
            expect(service.getAll().totalSeconds).toBe(7);
            interval.flush(2000);
            expect(service.getAll().startTime).toBe(10);
            expect(service.getAll().totalSeconds).toBe(5);
        });

        it('should stop existing timer if called twice', function () {
            service.startTimer({
                startTime: 10,
                totalSeconds: 10
            });
            expect(service.getAll().startTime).toBe(10);
            expect(service.getAll().totalSeconds).toBe(10);
            service.startTimer({
                startTime: 15,
                totalSeconds: 15
            });
            expect(service.getAll().startTime).toBe(15);
            expect(service.getAll().totalSeconds).toBe(15);
        })
    });

    describe('stopTimer', function () {
        it('should stop timer and set time to undefined when called', function () {
            service.startTimer({
                startTime: 10,
                totalSeconds: 10
            });
            expect(service.getAll().startTime).toBe(10);
            expect(service.getAll().totalSeconds).toBe(10);
            service.stopTimer();
            expect(service.getAll()).toBeFalsy();
        })
    });

    describe('getters', function () {
        it('should calculate correct minutes left', function () {
            service.startTimer({
                startTime: 59,
                totalSeconds: 59
            });
            expect(service.getMinutes()).toBe(0);

            service.startTimer({
                startTime: 60,
                totalSeconds: 60
            });
            expect(service.getMinutes()).toBe(1);

            service.startTimer({
                startTime: 61,
                totalSeconds: 61
            });
            expect(service.getMinutes()).toBe(1);

            service.startTimer({
                startTime: 121,
                totalSeconds: 121
            });
            expect(service.getMinutes()).toBe(2);
        });

        it('should calculate correct seconds left', function () {
            service.startTimer({
                startTime: 0,
                totalSeconds: 0
            });
            expect(service.getSeconds()).toBe(0);

            service.startTimer({
                startTime: 1,
                totalSeconds: 1
            });
            expect(service.getSeconds()).toBe(1);

            service.startTimer({
                startTime: 60,
                totalSeconds: 60
            });
            expect(service.getSeconds()).toBe(0);

            service.startTimer({
                startTime: 61,
                totalSeconds: 61
            });
            expect(service.getSeconds()).toBe(1);

            service.startTimer({
                startTime: 119,
                totalSeconds: 119
            });
            expect(service.getSeconds()).toBe(59);
        });

        it('should calculate correct percentage', function () {
            service.startTimer({
                startTime: 10,
                totalSeconds: 10
            });
            expect(service.getPercent()).toBe(100);
            interval.flush(3000);
            expect(service.getPercent()).toBe(70);
            interval.flush(3000);
            expect(service.getPercent()).toBe(40);
            interval.flush(4000);
            expect(service.getPercent()).toBe(0);
        })
    })
});